@@ -17,7 +17,6 @@ on_button_press (GooCanvasItem *item,
 		 gpointer data)
 {
   GooCanvasItem *parent1, *parent2, *parent;
-  int child_num;
 
   if (event->button != 1 || event->type != GDK_BUTTON_PRESS)
     return FALSE;
@@ -29,8 +28,7 @@ on_button_press (GooCanvasItem *item,
 
   parent = goo_canvas_item_get_parent (item);
   g_object_ref (item);
-  child_num = goo_canvas_item_find_child (parent, item);
-  goo_canvas_item_remove_child (parent, child_num);
+  goo_canvas_item_remove (item);
   if (parent == parent1)
     goo_canvas_item_add_child (parent2, item, -1);
   else
@@ -79,7 +77,7 @@ create_canvas_features (void)
 	root = goo_canvas_get_root_item (GOO_CANVAS (canvas));
 
 	gtk_widget_set_size_request (canvas, 400, 200);
-	goo_canvas_set_bounds (GOO_CANVAS (canvas), 0, 0, 300, 200);
+	goo_canvas_set_bounds (GOO_CANVAS (canvas), 0, 0, 400, 200);
 	gtk_container_add (GTK_CONTAINER (frame), canvas);
 	gtk_widget_show (canvas);
 
