@@ -17,10 +17,10 @@
 #include <goocanvas.h>
 #include "demo-item.h"
 
-GooCanvasItem *ellipse2, *textitem;
+static GooCanvasItem *ellipse2, *textitem;
 
-gboolean dragging = FALSE;
-double drag_x, drag_y;
+static gboolean dragging = FALSE;
+static double drag_x, drag_y;
 
 static void setup_canvas (GooCanvas *canvas);
 GtkWidget *create_canvas_fifteen (void);
@@ -33,12 +33,15 @@ GtkWidget *create_paths_page (void);
 GtkWidget *create_focus_page (void);
 GtkWidget *create_animation_page (void);
 GtkWidget *create_clipping_page (void);
+GtkWidget *create_table_page (void);
+GtkWidget *create_large_items_page (void);
 
 #if CAIRO_HAS_PDF_SURFACE
 static void
 write_pdf_clicked (GtkWidget *button, GooCanvas *canvas)
 {
   cairo_surface_t *surface;
+  GooCanvasBounds bounds;
   cairo_t *cr;
 
   g_print ("In write_pdf_clicked\n");
@@ -49,7 +52,16 @@ write_pdf_clicked (GtkWidget *button, GooCanvas *canvas)
   /* Place it in the middle of our 9x10 page. */
   cairo_translate (cr, 20, 130);
 
+  bounds.x1 = 100;
+  bounds.y1 = 100;
+  bounds.x2 = 300;
+  bounds.y2 = 300;
+
+#if 1
   goo_canvas_render (canvas, cr, NULL, 1.0);
+#else
+  goo_canvas_render (canvas, cr, &bounds, 1.0);
+#endif
 
   cairo_show_page (cr);
 
@@ -65,6 +77,24 @@ zoom_changed (GtkAdjustment *adj, GooCanvas *canvas)
 }
 
 
+static void
+zoom_x_changed (GtkAdjustment *adj, GooCanvas *canvas)
+{
+  g_object_set (G_OBJECT (canvas),
+		"scale-x", adj->value,
+		NULL);
+}
+
+
+static void
+zoom_y_changed (GtkAdjustment *adj, GooCanvas *canvas)
+{
+  g_object_set (G_OBJECT (canvas),
+		"scale-y", adj->value,
+		NULL);
+}
+
+
 static void
 center_toggled (GtkToggleButton *button, gpointer data)
 {
@@ -127,6 +157,44 @@ stop_animation_clicked (GtkWidget *button, GooCanvas *canvas)
 }
 
 
+static void
+change_bounds_clicked (GtkWidget *button, GooCanvas *canvas)
+{
+  static gdouble bounds[4][4] = {
+    { -100, -100, 200, 200 },
+    { 100, 100, 200, 200 },
+    { 0, 0, 300, 300 },
+    { 0, 0, 604, 454 }
+  };
+  static gint bounds_num = 0;
+
+  gboolean automatic_bounds = TRUE;
+  gboolean bounds_from_origin = FALSE;
+  gdouble bounds_padding = 50.0;
+
+  if (bounds_num < 4)
+    {
+      automatic_bounds = FALSE;
+      goo_canvas_set_bounds (canvas,
+			     bounds[bounds_num][0], bounds[bounds_num][1],
+			     bounds[bounds_num][2], bounds[bounds_num][3]);
+    }
+  else if (bounds_num == 4)
+    {
+      bounds_from_origin = TRUE;
+      bounds_padding = 50.0;
+    }
+
+  g_object_set (G_OBJECT (canvas),
+		"automatic-bounds", automatic_bounds,
+		"bounds-from-origin", bounds_from_origin,
+		"bounds-padding", bounds_padding,
+		NULL);
+
+  bounds_num = (bounds_num + 1) % 6;
+}
+
+
 static void
 move_ellipse_clicked (GtkWidget *button, GooCanvas *canvas)
 {
@@ -159,7 +227,7 @@ move_ellipse_clicked (GtkWidget *button, GooCanvas *canvas)
 		    "center-y", 150.0,
 		    "radius-x", 45.0,
 		    "radius-y", 40.0,
-		    "fill-color", "brown",
+		    "fill-pattern", NULL,
 		    "stroke-color", "midnightblue",
 		    "line-width", 4.0,
 		    "title", "A brown ellipse",
@@ -188,7 +256,7 @@ move_ellipse_clicked (GtkWidget *button, GooCanvas *canvas)
     }
   else if (last_state == 5)
     {
-      goo_canvas_item_set_simple_transform (ellipse2, -50, -50, 0.2, 225);
+      goo_canvas_item_set_simple_transform (ellipse2, 50, 50, 0.2, 225);
       last_state = 6;
     }
   else
@@ -231,6 +299,32 @@ on_motion_notify (GooCanvasItem *item,
 }
 
 
+static void
+output_items_in_area (GooCanvas *canvas,
+		      gdouble x,
+		      gdouble y)
+{
+  static gdouble last_x = 0.0, last_y = 0.0;
+  GooCanvasBounds area;
+  GList *items, *elem;
+
+  /* Print out the items from the last point to this one. */
+  area.x1 = MIN (x, last_x);
+  area.x2 = MAX (x, last_x);
+  area.y1 = MIN (y, last_y);
+  area.y2 = MAX (y, last_y);
+  items = goo_canvas_get_items_in_area (canvas, &area, TRUE, FALSE, FALSE);
+  for (elem = items; elem; elem = elem->next)
+    {
+      g_print ("  found items in area (%g, %g - %g, %g): %p\n",
+	       area.x1, area.y1, area.x2, area.y2, elem->data);
+    }
+  g_list_free (items);
+  last_x = x;
+  last_y = y;
+}
+
+
 static gboolean
 on_button_press (GooCanvasItem *item,
 		 GooCanvasItem *target,
@@ -239,23 +333,26 @@ on_button_press (GooCanvasItem *item,
 {
   GooCanvas *canvas;
   GdkCursor *fleur;
+  GList *items, *elem;
 
-#if 0
   g_print ("%p received 'button-press' signal at %g, %g (root: %g, %g)\n",
 	   item, event->x, event->y, event->x_root, event->y_root);
-#endif
+
+  canvas = goo_canvas_item_get_canvas (item);
+  items = goo_canvas_get_items_at (canvas, event->x_root, event->y_root,
+				   TRUE);
+  for (elem = items; elem; elem = elem->next)
+    g_print ("  found items: %p\n", elem->data);
+  g_list_free (items);
+
+  output_items_in_area (canvas, event->x_root, event->y_root);
 
   switch (event->button)
     {
     case 1:
       if (event->state & GDK_SHIFT_MASK)
 	{
-	  GooCanvasItem *parent;
-	  gint child_num;
-
-	  parent = goo_canvas_item_get_parent (item);
-	  child_num = goo_canvas_item_find_child (parent, item);
-	  goo_canvas_item_remove_child (parent, child_num);
+	  goo_canvas_item_remove (item);
 	}
       else
 	{
@@ -263,7 +360,6 @@ on_button_press (GooCanvasItem *item,
 	  drag_y = event->y;
 
 	  fleur = gdk_cursor_new (GDK_FLEUR);
-	  canvas = goo_canvas_item_get_canvas (item);
 	  goo_canvas_pointer_grab (canvas, item,
 				   GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
 				   fleur,
@@ -315,10 +411,15 @@ on_background_button_press (GooCanvasItem *item,
 			    GdkEventButton *event,
 			    gpointer data)
 {
+  GooCanvas *canvas;
+
 #if 1
   g_print ("background received 'button-press' signal\n");
 #endif
 
+  canvas = goo_canvas_item_get_canvas (item);
+  output_items_in_area (canvas, event->x_root, event->y_root);
+
   return TRUE;
 }
 
@@ -344,7 +445,6 @@ create_canvas_primitives ()
 	GtkWidget *scrolled_win, *canvas;
 	GtkAdjustment *adj;
 	GSList *group = NULL;
-	GdkColor bg_color = { 0, 50000, 50000, 65535 };
 
 	vbox = gtk_vbox_new (FALSE, 4);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
@@ -361,16 +461,21 @@ create_canvas_primitives ()
 	/* Create the canvas */
 
 	canvas = goo_canvas_new ();
-	gtk_widget_modify_base (canvas, GTK_STATE_NORMAL, &bg_color);
+	g_object_set (G_OBJECT (canvas),
+		      "automatic-bounds", TRUE,
+		      "bounds-from-origin", FALSE,
+		      "bounds-padding", 4.0,
+		      "background-color-rgb", 0xC3C3FF,
+		      NULL);
 	goo_canvas_set_bounds (GOO_CANVAS (canvas), 0, 0, 604, 454);
 
-	/* Zoom */
+	/* Scale */
 
-	w = gtk_label_new ("Zoom:");
+	w = gtk_label_new ("Scale:");
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 
-	adj = GTK_ADJUSTMENT (gtk_adjustment_new (1.00, 0.05, 100.00, 0.05, 0.50, 0.50));
+	adj = GTK_ADJUSTMENT (gtk_adjustment_new (1.00, 0.05, 50.00, 0.05, 0.50, 0.50));
 	w = gtk_spin_button_new (adj, 0.0, 2);
 	g_signal_connect (adj, "value_changed",
 			  (GtkSignalFunc) zoom_changed,
@@ -379,6 +484,39 @@ create_canvas_primitives ()
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 
+
+	w = gtk_label_new ("Scale X:");
+	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
+	gtk_widget_show (w);
+
+	adj = GTK_ADJUSTMENT (gtk_adjustment_new (1.00, 0.05, 50.00, 0.05, 0.50, 0.50));
+	w = gtk_spin_button_new (adj, 0.0, 2);
+	g_signal_connect (adj, "value_changed",
+			  (GtkSignalFunc) zoom_x_changed,
+			  canvas);
+	gtk_widget_set_size_request (w, 50, -1);
+	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
+	gtk_widget_show (w);
+
+
+	w = gtk_label_new ("Scale Y:");
+	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
+	gtk_widget_show (w);
+
+	adj = GTK_ADJUSTMENT (gtk_adjustment_new (1.00, 0.05, 50.00, 0.05, 0.50, 0.50));
+	w = gtk_spin_button_new (adj, 0.0, 2);
+	g_signal_connect (adj, "value_changed",
+			  (GtkSignalFunc) zoom_y_changed,
+			  canvas);
+	gtk_widget_set_size_request (w, 50, -1);
+	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
+	gtk_widget_show (w);
+
+
+	hbox = gtk_hbox_new (FALSE, 4);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show (hbox);
+
 	/* Center: */
 	
 	w = gtk_check_button_new_with_label("Center scroll region");
@@ -427,6 +565,13 @@ create_canvas_primitives ()
 			  canvas);
 #endif
 
+	w = gtk_button_new_with_label("Change Bounds");
+	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
+	gtk_widget_show (w);
+	g_signal_connect (w, "clicked",
+			  (GtkSignalFunc) change_bounds_clicked,
+			  canvas);
+
 	hbox = gtk_hbox_new (FALSE, 4);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 	gtk_widget_show (hbox);
@@ -685,6 +830,7 @@ setup_rectangles (GooCanvasItem *root)
 
   item = goo_canvas_rect_new (root, 10, 80, 70, 60,
 			      "fill-color", "steelblue",
+			      /*"fill-pattern", NULL,*/
 			      NULL);
   setup_item_signals (item);
 
@@ -1049,6 +1195,17 @@ setup_texts (GooCanvasItem *root)
 				  NULL);
   setup_item_signals (textitem);
 #endif
+
+#if 1
+  textitem = goo_canvas_text_new (root,
+				  "Ellipsized text.",
+				  20, 420, 115, GTK_ANCHOR_W,
+				  "font", "Sans 12",
+				  "fill_color", "blue",
+				  "ellipsize", PANGO_ELLIPSIZE_END,
+				  NULL);
+  setup_item_signals (textitem);
+#endif
 }
 
 
@@ -1182,7 +1339,8 @@ on_delete_event (GtkWidget *window,
 		 GdkEvent *event,
 		 gpointer unused_data)
 {
-  exit (0);
+  gtk_main_quit ();
+  return FALSE;
 }
 
 
@@ -1201,9 +1359,11 @@ create_window ()
   gtk_widget_show (notebook);
   gtk_container_add (GTK_CONTAINER (window), notebook);
 
+#if 1
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
 			    create_canvas_primitives (),
 			    gtk_label_new ("Primitives"));
+#endif
 #if 1
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
 			    create_canvas_arrowhead (),
@@ -1254,6 +1414,16 @@ create_window ()
 			    create_clipping_page (),
 			    gtk_label_new ("Clipping"));
 #endif
+#if 1
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
+			    create_table_page (),
+			    gtk_label_new ("Table"));
+#endif
+#if 1
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
+			    create_large_items_page (),
+			    gtk_label_new ("Large Items"));
+#endif
 
   return window;
 }
