@@ -16,10 +16,10 @@
 #endif
 #include <goocanvas.h>
 
-GooCanvasItemModel *ellipse2, *textitem;
+static GooCanvasItemModel *ellipse2, *textitem;
 
-gboolean dragging = FALSE;
-double drag_x, drag_y;
+static gboolean dragging = FALSE;
+static double drag_x, drag_y;
 
 GtkWidget *create_canvas_fifteen (void);
 GtkWidget *create_canvas_features (void);
@@ -31,6 +31,7 @@ GtkWidget *create_paths_page (void);
 GtkWidget *create_focus_page (void);
 GtkWidget *create_animation_page (void);
 GtkWidget *create_clipping_page (void);
+GtkWidget *create_table_page (void);
 
 #if CAIRO_HAS_PDF_SURFACE
 static void
@@ -125,6 +126,44 @@ stop_animation_clicked (GtkWidget *button, GooCanvas *canvas)
 }
 
 
+static void
+change_bounds_clicked (GtkWidget *button, GooCanvas *canvas)
+{
+  static gdouble bounds[4][4] = {
+    { -100, -100, 200, 200 },
+    { 100, 100, 200, 200 },
+    { 0, 0, 300, 300 },
+    { 0, 0, 604, 454 }
+  };
+  static gint bounds_num = 0;
+
+  gboolean automatic_bounds = TRUE;
+  gboolean bounds_from_origin = FALSE;
+  gdouble bounds_padding = 50.0;
+
+  if (bounds_num < 4)
+    {
+      automatic_bounds = FALSE;
+      goo_canvas_set_bounds (canvas,
+			     bounds[bounds_num][0], bounds[bounds_num][1],
+			     bounds[bounds_num][2], bounds[bounds_num][3]);
+    }
+  else if (bounds_num == 4)
+    {
+      bounds_from_origin = TRUE;
+      bounds_padding = 50.0;
+    }
+
+  g_object_set (G_OBJECT (canvas),
+		"automatic-bounds", automatic_bounds,
+		"bounds-from-origin", bounds_from_origin,
+		"bounds-padding", bounds_padding,
+		NULL);
+
+  bounds_num = (bounds_num + 1) % 6;
+}
+
+
 static void
 move_ellipse_clicked (GtkWidget *button, GooCanvas *canvas)
 {
@@ -164,6 +203,31 @@ move_ellipse_clicked (GtkWidget *button, GooCanvas *canvas)
 		    NULL);
       last_state = 2;
     }
+  else if (last_state == 2)
+    {
+      g_object_set (ellipse2,
+		    "center-x", 0.0,
+		    "center-y", 0.0,
+		    "radius-y", 30.0,
+		    NULL);
+      goo_canvas_item_model_set_simple_transform (ellipse2, 100, 100, 1, 0);
+      last_state = 3;
+    }
+  else if (last_state == 3)
+    {
+      goo_canvas_item_model_set_simple_transform (ellipse2, 200, 100, 2, 0);
+      last_state = 4;
+    }
+  else if (last_state == 4)
+    {
+      goo_canvas_item_model_set_simple_transform (ellipse2, 200, 200, 1, 45);
+      last_state = 5;
+    }
+  else if (last_state == 5)
+    {
+      goo_canvas_item_model_set_simple_transform (ellipse2, 50, 50, 0.2, 225);
+      last_state = 6;
+    }
   else
     {
       g_object_set (ellipse2,
@@ -229,12 +293,7 @@ on_button_press (GooCanvasItem *item,
     case 1:
       if (event->state & GDK_SHIFT_MASK)
 	{
-	  GooCanvasItemModel *parent;
-	  gint child_num;
-
-	  parent = goo_canvas_item_model_get_parent (model);
-	  child_num = goo_canvas_item_model_find_child (parent, model);
-	  goo_canvas_item_model_remove_child (parent, child_num);
+	  goo_canvas_item_model_remove (model);
 	}
       else
 	{
@@ -334,7 +393,6 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	GtkWidget *scrolled_win, *canvas;
 	GtkAdjustment *adj;
 	GSList *group = NULL;
-	GdkColor bg_color = { 0, 50000, 50000, 65535 };
 
 	vbox = gtk_vbox_new (FALSE, 4);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
@@ -351,7 +409,12 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	/* Create the canvas */
 
 	canvas = goo_canvas_new ();
-	gtk_widget_modify_base (canvas, GTK_STATE_NORMAL, &bg_color);
+	g_object_set (G_OBJECT (canvas),
+		      "automatic-bounds", TRUE,
+		      "bounds-from-origin", FALSE,
+		      "bounds-padding", 4.0,
+		      "background-color-rgb", 0xC3C3FF,
+		      NULL);
 	goo_canvas_set_bounds (GOO_CANVAS (canvas), 0, 0, 604, 454);
 
 	g_signal_connect (canvas, "item_created",
@@ -363,7 +426,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 
-	adj = GTK_ADJUSTMENT (gtk_adjustment_new (1.00, 0.05, 100.00, 0.05, 0.50, 0.50));
+	adj = GTK_ADJUSTMENT (gtk_adjustment_new (1.00, 0.05, 50.00, 0.05, 0.50, 0.50));
 	w = gtk_spin_button_new (adj, 0.0, 2);
 	g_signal_connect (adj, "value_changed",
 			  (GtkSignalFunc) zoom_changed,
@@ -420,6 +483,13 @@ create_canvas_primitives (GooCanvasItemModel *model)
 			  canvas);
 #endif
 
+	w = gtk_button_new_with_label("Change Bounds");
+	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
+	gtk_widget_show (w);
+	g_signal_connect (w, "clicked",
+			  (GtkSignalFunc) change_bounds_clicked,
+			  canvas);
+
 	hbox = gtk_hbox_new (FALSE, 4);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 	gtk_widget_show (hbox);
@@ -1026,6 +1096,16 @@ setup_texts (GooCanvasItemModel *root)
 					"fill_color", "goldenrod",
 					NULL);
 #endif
+
+#if 1
+  textitem = goo_canvas_text_model_new (root,
+					"Ellipsized text.",
+					20, 420, 115, GTK_ANCHOR_W,
+					"font", "Sans 12",
+					"fill_color", "blue",
+					"ellipsize", PANGO_ELLIPSIZE_END,
+					NULL);
+#endif
 }
 
 
@@ -1162,7 +1242,8 @@ on_delete_event (GtkWidget *window,
 		 GdkEvent *event,
 		 gpointer unused_data)
 {
-  exit (0);
+  gtk_main_quit ();
+  return FALSE;
 }
 
 
@@ -1234,6 +1315,11 @@ create_window (GooCanvasItemModel *model)
 			    create_clipping_page (),
 			    gtk_label_new ("Clipping"));
 #endif
+#if 1
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
+			    create_table_page (),
+			    gtk_label_new ("Table"));
+#endif
 
   return window;
 }
