@@ -41,6 +41,7 @@
 #include "goocanvasatk.h"
 
 
+static void goo_canvas_group_dispose	  (GObject            *object);
 static void goo_canvas_group_finalize     (GObject            *object);
 static void canvas_item_interface_init    (GooCanvasItemIface *iface);
 
@@ -55,6 +56,7 @@ goo_canvas_group_class_init (GooCanvasGroupClass *klass)
 {
   GObjectClass *gobject_class = (GObjectClass*) klass;
 
+  gobject_class->dispose  = goo_canvas_group_dispose;
   gobject_class->finalize = goo_canvas_group_finalize;
 
   /* Register our accessible factory, but only if accessibility is enabled. */
@@ -114,7 +116,7 @@ goo_canvas_group_new (GooCanvasItem *parent,
 
 
 static void
-goo_canvas_group_finalize (GObject *object)
+goo_canvas_group_dispose (GObject *object)
 {
   GooCanvasGroup *group = (GooCanvasGroup*) object;
   gint i;
@@ -127,6 +129,17 @@ goo_canvas_group_finalize (GObject *object)
       g_object_unref (item);
     }
 
+  g_ptr_array_set_size (group->items, 0);
+
+  G_OBJECT_CLASS (goo_canvas_group_parent_class)->dispose (object);
+}
+
+
+static void
+goo_canvas_group_finalize (GObject *object)
+{
+  GooCanvasGroup *group = (GooCanvasGroup*) object;
+
   g_ptr_array_free (group->items, TRUE);
 
   G_OBJECT_CLASS (goo_canvas_group_parent_class)->finalize (object);
@@ -191,6 +204,8 @@ goo_canvas_group_remove_child  (GooCanvasItem  *item,
   GooCanvasItem *child;
   GooCanvasBounds bounds;
 
+  g_return_if_fail (child_num < group->items->len);
+
   /* Request a redraw of the item's bounds. */
   child = group->items->pdata[child_num];
   if (simple->canvas)
@@ -393,24 +408,29 @@ goo_canvas_group_update  (GooCanvasItem   *item,
 }
 
 
-static GooCanvasItem*
-goo_canvas_group_get_item_at (GooCanvasItem  *item,
-			      gdouble         x,
-			      gdouble         y,
-			      cairo_t        *cr,
-			      gboolean        is_pointer_event,
-			      gboolean        parent_visible)
+static GList*
+goo_canvas_group_get_items_at (GooCanvasItem  *item,
+			       gdouble         x,
+			       gdouble         y,
+			       cairo_t        *cr,
+			       gboolean        is_pointer_event,
+			       gboolean        parent_visible,
+			       GList          *found_items)
 {
   GooCanvasItemSimple *simple = (GooCanvasItemSimple*) item;
   GooCanvasItemSimpleData *simple_data = simple->simple_data;
   GooCanvasGroup *group = (GooCanvasGroup*) item;
-  GooCanvasItem *found_item = NULL;
   gboolean visible = parent_visible;
   int i;
 
   if (simple->need_update)
     goo_canvas_item_ensure_updated (item);
 
+  /* Skip the item if the point isn't in the item's bounds. */
+  if (simple->bounds.x1 > x || simple->bounds.x2 < x
+      || simple->bounds.y1 > y || simple->bounds.y2 < y)
+    return found_items;
+
   if (simple_data->visibility <= GOO_CANVAS_ITEM_INVISIBLE
       || (simple_data->visibility == GOO_CANVAS_ITEM_VISIBLE_ABOVE_THRESHOLD
 	  && simple->canvas->scale < simple_data->visibility_threshold))
@@ -421,40 +441,59 @@ goo_canvas_group_get_item_at (GooCanvasItem  *item,
       && (simple_data->pointer_events == GOO_CANVAS_EVENTS_NONE
 	  || ((simple_data->pointer_events & GOO_CANVAS_EVENTS_VISIBLE_MASK)
 	      && !visible)))
-    return NULL;
+    return found_items;
 
-  /* Step down from the top item to the bottom in the stack/layer, and return
-     the first item found that contains the given point. */
   cairo_save (cr);
   if (simple_data->transform)
     cairo_transform (cr, simple_data->transform);
 
-  for (i = group->items->len - 1; i >= 0; i--)
+  /* If the group has a clip path, check if the point is inside it. */
+  if (simple_data->clip_path_commands)
+    {
+      double user_x = x, user_y = y;
+
+      cairo_device_to_user (cr, &user_x, &user_y);
+      goo_canvas_create_path (simple_data->clip_path_commands, cr);
+      cairo_set_fill_rule (cr, simple_data->clip_fill_rule);
+      if (!cairo_in_fill (cr, user_x, user_y))
+	{
+	  cairo_restore (cr);
+	  return found_items;
+	}
+    }
+
+  /* Step up from the bottom of the children to the top, adding any items
+     found to the start of the list. */
+  for (i = 0; i < group->items->len; i++)
     {
       GooCanvasItem *child = group->items->pdata[i];
 
-      found_item = goo_canvas_item_get_item_at (child, x, y, cr,
-						is_pointer_event, visible);
-      if (found_item)
-	break;
+      found_items = goo_canvas_item_get_items_at (child, x, y, cr,
+						  is_pointer_event, visible,
+						  found_items);
     }
   cairo_restore (cr);
 
-  return found_item;
+  return found_items;
 }
 
 
 static void
-goo_canvas_group_paint (GooCanvasItem     *item,
-			cairo_t           *cr,
-			GooCanvasBounds   *bounds,
-			gdouble            scale)
+goo_canvas_group_paint (GooCanvasItem         *item,
+			cairo_t               *cr,
+			const GooCanvasBounds *bounds,
+			gdouble                scale)
 {
   GooCanvasItemSimple *simple = (GooCanvasItemSimple*) item;
   GooCanvasItemSimpleData *simple_data = simple->simple_data;
   GooCanvasGroup *group = (GooCanvasGroup*) item;
   gint i;
 
+  /* Skip the item if the bounds don't intersect the expose rectangle. */
+  if (simple->bounds.x1 > bounds->x2 || simple->bounds.x2 < bounds->x1
+      || simple->bounds.y1 > bounds->y2 || simple->bounds.y2 < bounds->y1)
+    return;
+
   /* Check if the item should be visible. */
   if (simple_data->visibility <= GOO_CANVAS_ITEM_INVISIBLE
       || (simple_data->visibility == GOO_CANVAS_ITEM_VISIBLE_ABOVE_THRESHOLD
@@ -466,6 +505,14 @@ goo_canvas_group_paint (GooCanvasItem     *item,
   if (simple_data->transform)
     cairo_transform (cr, simple_data->transform);
 
+  /* Clip with the group's clip path, if it is set. */
+  if (simple_data->clip_path_commands)
+    {
+      goo_canvas_create_path (simple_data->clip_path_commands, cr);
+      cairo_set_fill_rule (cr, simple_data->clip_fill_rule);
+      cairo_clip (cr);
+    }
+
   for (i = 0; i < group->items->len; i++)
     {
       GooCanvasItem *child = group->items->pdata[i];
@@ -487,7 +534,7 @@ canvas_item_interface_init (GooCanvasItemIface *iface)
   iface->move_child     = goo_canvas_group_move_child;
   iface->remove_child   = goo_canvas_group_remove_child;
 
-  iface->get_item_at	= goo_canvas_group_get_item_at;
+  iface->get_items_at	= goo_canvas_group_get_items_at;
   iface->update         = goo_canvas_group_update;
   iface->paint          = goo_canvas_group_paint;
 
@@ -525,6 +572,7 @@ canvas_item_interface_init (GooCanvasItemIface *iface)
  * (See goo_canvas_get_item() and #GooCanvas::item-created.)
  */
 static void item_model_interface_init (GooCanvasItemModelIface *iface);
+static void goo_canvas_group_model_dispose      (GObject            *object);
 static void goo_canvas_group_model_finalize     (GObject            *object);
 
 G_DEFINE_TYPE_WITH_CODE (GooCanvasGroupModel, goo_canvas_group_model,
@@ -538,6 +586,7 @@ goo_canvas_group_model_class_init (GooCanvasGroupModelClass *klass)
 {
   GObjectClass *gobject_class = (GObjectClass*) klass;
 
+  gobject_class->dispose  = goo_canvas_group_model_dispose;
   gobject_class->finalize = goo_canvas_group_model_finalize;
 }
 
@@ -589,7 +638,7 @@ goo_canvas_group_model_new (GooCanvasItemModel *parent,
 
 
 static void
-goo_canvas_group_model_finalize (GObject *object)
+goo_canvas_group_model_dispose (GObject *object)
 {
   GooCanvasGroupModel *gmodel = (GooCanvasGroupModel*) object;
   gint i;
@@ -602,12 +651,26 @@ goo_canvas_group_model_finalize (GObject *object)
       g_object_unref (child);
     }
 
+  g_ptr_array_set_size (gmodel->children, 0);
+
+  G_OBJECT_CLASS (goo_canvas_group_model_parent_class)->dispose (object);
+}
+
+
+static void
+goo_canvas_group_model_finalize (GObject *object)
+{
+  GooCanvasGroupModel *gmodel = (GooCanvasGroupModel*) object;
+
   g_ptr_array_free (gmodel->children, TRUE);
 
   G_OBJECT_CLASS (goo_canvas_group_model_parent_class)->finalize (object);
 }
 
 
+extern void _goo_canvas_item_model_emit_child_added (GooCanvasItemModel *model,
+						     gint                position);
+
 static void
 goo_canvas_group_model_add_child     (GooCanvasItemModel *model,
 				      GooCanvasItemModel *child,
@@ -629,7 +692,7 @@ goo_canvas_group_model_add_child     (GooCanvasItemModel *model,
 
   goo_canvas_item_model_set_parent (child, model);
 
-  g_signal_emit_by_name (gmodel, "child-added", position);
+  _goo_canvas_item_model_emit_child_added (model, position);
 }
 
 
