@@ -131,7 +131,7 @@ goo_canvas_item_base_init (gpointer g_iface)
 		      G_SIGNAL_RUN_LAST,
 		      G_STRUCT_OFFSET (GooCanvasItemIface,
 				       enter_notify_event),
-		      NULL, NULL,
+		      goo_canvas_boolean_handled_accumulator, NULL,
 		      goo_canvas_marshal_BOOLEAN__OBJECT_BOXED,
 		      G_TYPE_BOOLEAN, 2,
 		      GOO_TYPE_CANVAS_ITEM,
@@ -155,7 +155,7 @@ goo_canvas_item_base_init (gpointer g_iface)
 		      G_SIGNAL_RUN_LAST,
 		      G_STRUCT_OFFSET (GooCanvasItemIface,
 				       leave_notify_event),
-		      NULL, NULL,
+		      goo_canvas_boolean_handled_accumulator, NULL,
 		      goo_canvas_marshal_BOOLEAN__OBJECT_BOXED,
 		      G_TYPE_BOOLEAN, 2,
 		      GOO_TYPE_CANVAS_ITEM,
@@ -179,7 +179,7 @@ goo_canvas_item_base_init (gpointer g_iface)
 		      G_SIGNAL_RUN_LAST,
 		      G_STRUCT_OFFSET (GooCanvasItemIface,
 				       motion_notify_event),
-		      NULL, NULL,
+		      goo_canvas_boolean_handled_accumulator, NULL,
 		      goo_canvas_marshal_BOOLEAN__OBJECT_BOXED,
 		      G_TYPE_BOOLEAN, 2,
 		      GOO_TYPE_CANVAS_ITEM,
@@ -203,7 +203,7 @@ goo_canvas_item_base_init (gpointer g_iface)
 		      G_SIGNAL_RUN_LAST,
 		      G_STRUCT_OFFSET (GooCanvasItemIface,
 				       button_press_event),
-		      NULL, NULL,
+		      goo_canvas_boolean_handled_accumulator, NULL,
 		      goo_canvas_marshal_BOOLEAN__OBJECT_BOXED,
 		      G_TYPE_BOOLEAN, 2,
 		      GOO_TYPE_CANVAS_ITEM,
@@ -227,7 +227,7 @@ goo_canvas_item_base_init (gpointer g_iface)
 		      G_SIGNAL_RUN_LAST,
 		      G_STRUCT_OFFSET (GooCanvasItemIface,
 				       button_release_event),
-		      NULL, NULL,
+		      goo_canvas_boolean_handled_accumulator, NULL,
 		      goo_canvas_marshal_BOOLEAN__OBJECT_BOXED,
 		      G_TYPE_BOOLEAN, 2,
 		      GOO_TYPE_CANVAS_ITEM,
@@ -253,7 +253,7 @@ goo_canvas_item_base_init (gpointer g_iface)
 		      G_SIGNAL_RUN_LAST,
 		      G_STRUCT_OFFSET (GooCanvasItemIface,
 				       focus_in_event),
-		      NULL, NULL,
+		      goo_canvas_boolean_handled_accumulator, NULL,
 		      goo_canvas_marshal_BOOLEAN__OBJECT_BOXED,
 		      G_TYPE_BOOLEAN, 2,
 		      GOO_TYPE_CANVAS_ITEM,
@@ -276,7 +276,7 @@ goo_canvas_item_base_init (gpointer g_iface)
 		      G_SIGNAL_RUN_LAST,
 		      G_STRUCT_OFFSET (GooCanvasItemIface,
 				       focus_out_event),
-		      NULL, NULL,
+		      goo_canvas_boolean_handled_accumulator, NULL,
 		      goo_canvas_marshal_BOOLEAN__OBJECT_BOXED,
 		      G_TYPE_BOOLEAN, 2,
 		      GOO_TYPE_CANVAS_ITEM,
@@ -300,7 +300,7 @@ goo_canvas_item_base_init (gpointer g_iface)
 		      G_SIGNAL_RUN_LAST,
 		      G_STRUCT_OFFSET (GooCanvasItemIface,
 				       key_press_event),
-		      NULL, NULL,
+		      goo_canvas_boolean_handled_accumulator, NULL,
 		      goo_canvas_marshal_BOOLEAN__OBJECT_BOXED,
 		      G_TYPE_BOOLEAN, 2,
 		      GOO_TYPE_CANVAS_ITEM,
@@ -324,7 +324,7 @@ goo_canvas_item_base_init (gpointer g_iface)
 		      G_SIGNAL_RUN_LAST,
 		      G_STRUCT_OFFSET (GooCanvasItemIface,
 				       key_release_event),
-		      NULL, NULL,
+		      goo_canvas_boolean_handled_accumulator, NULL,
 		      goo_canvas_marshal_BOOLEAN__OBJECT_BOXED,
 		      G_TYPE_BOOLEAN, 2,
 		      GOO_TYPE_CANVAS_ITEM,
@@ -349,7 +349,7 @@ goo_canvas_item_base_init (gpointer g_iface)
 		      G_SIGNAL_RUN_LAST,
 		      G_STRUCT_OFFSET (GooCanvasItemIface,
 				       grab_broken_event),
-		      NULL, NULL,
+		      goo_canvas_boolean_handled_accumulator, NULL,
 		      goo_canvas_marshal_BOOLEAN__OBJECT_BOXED,
 		      G_TYPE_BOOLEAN, 2,
 		      GOO_TYPE_CANVAS_ITEM,
@@ -474,10 +474,10 @@ goo_canvas_item_get_canvas (GooCanvasItem *item)
  * @item: a #GooCanvasItem.
  * @canvas: a #GooCanvas
  * 
- * Sets the canvas of the item.
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically container items such as #GooCanvasGroup.
  *
- * This is only needed for setting the canvas of the root item, as the other
- * items can get the canvas from their parents.
+ * It sets the canvas of the item.
  **/
 void
 goo_canvas_item_set_canvas     (GooCanvasItem   *item,
@@ -655,7 +655,10 @@ goo_canvas_item_get_parent  (GooCanvasItem *item)
  * @item: an item.
  * @parent: the new parent item.
  * 
- * Sets the parent of the item.
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically container items such as #GooCanvasGroup.
+ *
+ * It sets the parent of the item.
  **/
 void
 goo_canvas_item_set_parent (GooCanvasItem *item,
@@ -665,6 +668,33 @@ goo_canvas_item_set_parent (GooCanvasItem *item,
 }
 
 
+/**
+ * goo_canvas_item_remove:
+ * @item: an item.
+ * 
+ * Removes an item from its parent. If the item is in a canvas it will be
+ * removed.
+ *
+ * This would normally also result in the item being freed.
+ **/
+void
+goo_canvas_item_remove (GooCanvasItem *item)
+{
+  GooCanvasItem *parent;
+  gint child_num;
+
+  parent = goo_canvas_item_get_parent (item);
+  if (!parent)
+    return;
+
+  child_num = goo_canvas_item_find_child (parent, item);
+  if (child_num == -1)
+    return;
+
+  goo_canvas_item_remove_child (parent, child_num);
+}
+
+
 /**
  * goo_canvas_item_raise:
  * @item: an item.
@@ -754,17 +784,19 @@ goo_canvas_item_lower          (GooCanvasItem *item,
 /**
  * goo_canvas_item_get_transform:
  * @item: an item.
+ * @transform: the place to store the transform.
  * 
  * Gets the transformation matrix of an item.
  * 
- * Returns: the item's transformation matrix.
+ * Returns: %TRUE if a transform is set.
  **/
-cairo_matrix_t*
-goo_canvas_item_get_transform  (GooCanvasItem *item)
+gboolean
+goo_canvas_item_get_transform  (GooCanvasItem   *item,
+				cairo_matrix_t  *transform)
 {
   GooCanvasItemIface *iface = GOO_CANVAS_ITEM_GET_IFACE (item);
 
-  return iface->get_transform ? iface->get_transform (item) : NULL;
+  return iface->get_transform ? iface->get_transform (item, transform) : FALSE;
 }
 
 
@@ -786,21 +818,13 @@ goo_canvas_item_get_transform_for_child  (GooCanvasItem  *item,
 					  cairo_matrix_t *transform)
 {
   GooCanvasItemIface *iface = GOO_CANVAS_ITEM_GET_IFACE (item);
-  cairo_matrix_t *item_transform;
 
   if (child && iface->get_transform_for_child)
     return iface->get_transform_for_child (item, child, transform);
 
   /* We fallback to the standard get_transform method. */
   if (iface->get_transform)
-    {
-      item_transform = iface->get_transform (item);
-      if (item_transform)
-	{
-	  *transform = *item_transform;
-	  return TRUE;
-	}
-    }
+    return iface->get_transform (item, transform);
 
   return FALSE;
 }
@@ -809,16 +833,16 @@ goo_canvas_item_get_transform_for_child  (GooCanvasItem  *item,
 /**
  * goo_canvas_item_set_transform:
  * @item: an item.
- * @matrix: the new transformation matrix, or %NULL to reset the
+ * @transform: the new transformation matrix, or %NULL to reset the
  *  transformation to the identity matrix.
  * 
  * Sets the transformation matrix of an item.
  **/
 void
-goo_canvas_item_set_transform  (GooCanvasItem *item,
-				cairo_matrix_t *matrix)
+goo_canvas_item_set_transform  (GooCanvasItem        *item,
+				const cairo_matrix_t *transform)
 {
-  GOO_CANVAS_ITEM_GET_IFACE (item)->set_transform (item, matrix);
+  GOO_CANVAS_ITEM_GET_IFACE (item)->set_transform (item, transform);
 }
 
 
@@ -863,11 +887,9 @@ goo_canvas_item_translate      (GooCanvasItem *item,
 				gdouble        ty)
 {
   GooCanvasItemIface *iface = GOO_CANVAS_ITEM_GET_IFACE (item);
-  cairo_matrix_t *matrix, new_matrix = { 1, 0, 0, 1, 0, 0 };
+  cairo_matrix_t new_matrix = { 1, 0, 0, 1, 0, 0 };
 
-  matrix = iface->get_transform (item);
-  if (matrix)
-    new_matrix = *matrix;
+  iface->get_transform (item, &new_matrix);
   cairo_matrix_translate (&new_matrix, tx, ty);
   iface->set_transform (item, &new_matrix);
 }
@@ -887,11 +909,9 @@ goo_canvas_item_scale          (GooCanvasItem *item,
 				gdouble        sy)
 {
   GooCanvasItemIface *iface = GOO_CANVAS_ITEM_GET_IFACE (item);
-  cairo_matrix_t *matrix, new_matrix = { 1, 0, 0, 1, 0, 0 };
+  cairo_matrix_t new_matrix = { 1, 0, 0, 1, 0, 0 };
 
-  matrix = iface->get_transform (item);
-  if (matrix)
-    new_matrix = *matrix;
+  iface->get_transform (item, &new_matrix);
   cairo_matrix_scale (&new_matrix, sx, sy);
   iface->set_transform (item, &new_matrix);
 }
@@ -914,12 +934,10 @@ goo_canvas_item_rotate         (GooCanvasItem *item,
 				gdouble        cy)
 {
   GooCanvasItemIface *iface = GOO_CANVAS_ITEM_GET_IFACE (item);
-  cairo_matrix_t *matrix, new_matrix = { 1, 0, 0, 1, 0, 0 };
+  cairo_matrix_t new_matrix = { 1, 0, 0, 1, 0, 0 };
   double radians = degrees * (M_PI / 180);
 
-  matrix = iface->get_transform (item);
-  if (matrix)
-    new_matrix = *matrix;
+  iface->get_transform (item, &new_matrix);
   cairo_matrix_translate (&new_matrix, cx, cy);
   cairo_matrix_rotate (&new_matrix, radians);
   cairo_matrix_translate (&new_matrix, -cx, -cy);
@@ -944,12 +962,10 @@ goo_canvas_item_skew_x         (GooCanvasItem *item,
 				gdouble        cy)
 {
   GooCanvasItemIface *iface = GOO_CANVAS_ITEM_GET_IFACE (item);
-  cairo_matrix_t *matrix, tmp, new_matrix = { 1, 0, 0, 1, 0, 0 };
+  cairo_matrix_t tmp, new_matrix = { 1, 0, 0, 1, 0, 0 };
   double radians = degrees * (M_PI / 180);
 
-  matrix = iface->get_transform (item);
-  if (matrix)
-    new_matrix = *matrix;
+  iface->get_transform (item, &new_matrix);
   cairo_matrix_translate (&new_matrix, cx, cy);
   cairo_matrix_init (&tmp, 1, 0, tan (radians), 1, 0, 0);
   cairo_matrix_multiply (&new_matrix, &tmp, &new_matrix);
@@ -975,12 +991,10 @@ goo_canvas_item_skew_y         (GooCanvasItem *item,
 				gdouble        cy)
 {
   GooCanvasItemIface *iface = GOO_CANVAS_ITEM_GET_IFACE (item);
-  cairo_matrix_t *matrix, tmp, new_matrix = { 1, 0, 0, 1, 0, 0 };
+  cairo_matrix_t tmp, new_matrix = { 1, 0, 0, 1, 0, 0 };
   double radians = degrees * (M_PI / 180);
 
-  matrix = iface->get_transform (item);
-  if (matrix)
-    new_matrix = *matrix;
+  iface->get_transform (item, &new_matrix);
   cairo_matrix_translate (&new_matrix, cx, cy);
   cairo_matrix_init (&tmp, 1, tan (radians), 0, 1, 0, 0);
   cairo_matrix_multiply (&new_matrix, &tmp, &new_matrix);
@@ -1057,8 +1071,11 @@ goo_canvas_item_free_animation (GooCanvasItemAnimation *anim)
 static gboolean
 goo_canvas_item_animate_cb (GooCanvasItemAnimation *anim)
 {
-  GooCanvasItemIface *iface;
-  GooCanvasItemModelIface *model_iface;
+  GooCanvasItem *item = anim->item;
+  GooCanvasItemModel *model = anim->model;
+  GooCanvasAnimateType type = anim->type;
+  GooCanvasItemIface *iface = NULL;
+  GooCanvasItemModelIface *model_iface = NULL;
   cairo_matrix_t new_matrix;
   gboolean keep_source = TRUE;
   gdouble scale;
@@ -1066,21 +1083,21 @@ goo_canvas_item_animate_cb (GooCanvasItemAnimation *anim)
 
   GDK_THREADS_ENTER ();
 
-  if (anim->model)
-    model_iface = GOO_CANVAS_ITEM_MODEL_GET_IFACE (anim->model);
+  if (model)
+    model_iface = GOO_CANVAS_ITEM_MODEL_GET_IFACE (model);
   else
-    iface = GOO_CANVAS_ITEM_GET_IFACE (anim->item);
+    iface = GOO_CANVAS_ITEM_GET_IFACE (item);
 
   if (++anim->step > anim->total_steps)
     {
-      switch (anim->type)
+      switch (type)
 	{
 	case GOO_CANVAS_ANIMATE_RESET:
 	  /* Reset the transform to the initial value. */
-	  if (anim->model)
-	    model_iface->set_transform (anim->model, &anim->start);
+	  if (model)
+	    model_iface->set_transform (model, &anim->start);
 	  else
-	    iface->set_transform (anim->item, &anim->start);
+	    iface->set_transform (item, &anim->start);
 
 	  /* Fall through.. */
 	case GOO_CANVAS_ANIMATE_FREEZE:
@@ -1088,10 +1105,10 @@ goo_canvas_item_animate_cb (GooCanvasItemAnimation *anim)
 	  anim->timeout_id = 0;
 	  /* This will result in a call to goo_canvas_item_free_animation()
 	     above. We've set the timeout_id to 0 so it isn't removed twice. */
-	  if (anim->model)
-	    g_object_set_data (G_OBJECT (anim->model), animation_key, NULL);
+	  if (model)
+	    g_object_set_data (G_OBJECT (model), animation_key, NULL);
 	  else
-	    g_object_set_data (G_OBJECT (anim->item), animation_key, NULL);
+	    g_object_set_data (G_OBJECT (item), animation_key, NULL);
 	  break;
 
 	case GOO_CANVAS_ANIMATE_RESTART:
@@ -1105,34 +1122,36 @@ goo_canvas_item_animate_cb (GooCanvasItemAnimation *anim)
 	}
     }
 
-  step = anim->forward ? anim->step : anim->total_steps - anim->step;
-
-  if (anim->absolute)
+  if (keep_source)
     {
-      cairo_matrix_init_identity (&new_matrix);
-      scale = anim->scale_start + anim->scale_step * step;
-      cairo_matrix_translate (&new_matrix,
-			      anim->x_start + anim->x_step * step,
-			      anim->y_start + anim->y_step * step);
-      cairo_matrix_scale (&new_matrix, scale, scale);
-      cairo_matrix_rotate (&new_matrix,
-			   anim->radians_start + anim->radians_step * step);
-    }
-  else
-    {
-      new_matrix = anim->start;
-      scale = 1 + anim->scale_step * step;
-      cairo_matrix_translate (&new_matrix, anim->x_step * step,
-			      anim->y_step * step);
-      cairo_matrix_scale (&new_matrix, scale, scale);
-      cairo_matrix_rotate (&new_matrix, anim->radians_step * step);
-    }
+      step = anim->forward ? anim->step : anim->total_steps - anim->step;
 
-  if (anim->model)
-    model_iface->set_transform (anim->model, &new_matrix);
-  else
-    iface->set_transform (anim->item, &new_matrix);
+      if (anim->absolute)
+	{
+	  cairo_matrix_init_identity (&new_matrix);
+	  scale = anim->scale_start + anim->scale_step * step;
+	  cairo_matrix_translate (&new_matrix,
+				  anim->x_start + anim->x_step * step,
+				  anim->y_start + anim->y_step * step);
+	  cairo_matrix_scale (&new_matrix, scale, scale);
+	  cairo_matrix_rotate (&new_matrix,
+			       anim->radians_start + anim->radians_step * step);
+	}
+      else
+	{
+	  new_matrix = anim->start;
+	  scale = 1 + anim->scale_step * step;
+	  cairo_matrix_translate (&new_matrix, anim->x_step * step,
+				  anim->y_step * step);
+	  cairo_matrix_scale (&new_matrix, scale, scale);
+	  cairo_matrix_rotate (&new_matrix, anim->radians_step * step);
+	}
 
+      if (model)
+	model_iface->set_transform (model, &new_matrix);
+      else
+	iface->set_transform (item, &new_matrix);
+    }
 
   GDK_THREADS_LEAVE ();
 
@@ -1154,19 +1173,19 @@ _goo_canvas_item_animate_internal (GooCanvasItem       *item,
 				   GooCanvasAnimateType type)
 {
   GObject *object;
-  cairo_matrix_t *matrix;
+  cairo_matrix_t matrix = { 1, 0, 0, 1, 0, 0 };
   GooCanvasItemAnimation *anim;
 
   if (item)
     {
       GooCanvasItemIface *iface = GOO_CANVAS_ITEM_GET_IFACE (item);
-      matrix = iface->get_transform (item);
+      iface->get_transform (item, &matrix);
       object = (GObject*) item;
     }
   else
     {
       GooCanvasItemModelIface *iface = GOO_CANVAS_ITEM_MODEL_GET_IFACE (model);
-      matrix = iface->get_transform (model);
+      iface->get_transform (model, &matrix);
       object = (GObject*) model;
     }
 
@@ -1176,10 +1195,7 @@ _goo_canvas_item_animate_internal (GooCanvasItem       *item,
   anim->model = model;
   anim->step = 0;
   anim->total_steps = duration / step_time;
-  if (matrix)
-    anim->start = *matrix;
-  else
-    cairo_matrix_init_identity (&anim->start);
+  anim->start = matrix;
   anim->absolute = absolute;
   anim->forward = TRUE;
 
@@ -1282,7 +1298,10 @@ goo_canvas_item_stop_animation (GooCanvasItem *item)
  * goo_canvas_item_request_update:
  * @item: a #GooCanvasItem.
  * 
- * Requests that an update of the item is scheduled. It will be performed
+ * This function is only intended to be used when implementing new canvas
+ * items.
+ *
+ * It requests that an update of the item is scheduled. It will be performed
  * as soon as the application is idle, and before the canvas is redrawn.
  **/
 void
@@ -1291,9 +1310,9 @@ goo_canvas_item_request_update  (GooCanvasItem *item)
   GooCanvasItemIface *iface = GOO_CANVAS_ITEM_GET_IFACE (item);
 
   if (iface->request_update)
-    return iface->request_update (item);
+    iface->request_update (item);
   else
-    return goo_canvas_item_request_update (iface->get_parent (item));
+    goo_canvas_item_request_update (iface->get_parent (item));
 }
 
 
@@ -1318,7 +1337,7 @@ goo_canvas_item_get_bounds  (GooCanvasItem   *item,
 
 
 /**
- * goo_canvas_item_get_item_at:
+ * goo_canvas_item_get_items_at:
  * @item: a #GooCanvasItem.
  * @x: the x coordinate of the point.
  * @y: the y coordinate of the point.
@@ -1327,23 +1346,32 @@ goo_canvas_item_get_bounds  (GooCanvasItem   *item,
  *  be used to determine which parts of the item are tested.
  * @parent_is_visible: %TRUE if the parent item is visible (which
  *  implies that all ancestors are also visible).
+ * @found_items: the list of items found so far.
  * 
- * Gets the item at the given point.
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically container items such as #GooCanvasGroup.
+ *
+ * It gets the items at the given point.
  * 
- * Returns: the item found at the given point, or %NULL if no item  was found.
+ * Returns: the @found_items list, with any more found items added onto
+ *  the start of the list, leaving the top item first.
  **/
-GooCanvasItem*
-goo_canvas_item_get_item_at (GooCanvasItem  *item,
-			     gdouble         x,
-			     gdouble         y,
-			     cairo_t        *cr,
-			     gboolean        is_pointer_event,
-			     gboolean        parent_is_visible)
+GList*
+goo_canvas_item_get_items_at (GooCanvasItem  *item,
+			      gdouble         x,
+			      gdouble         y,
+			      cairo_t        *cr,
+			      gboolean        is_pointer_event,
+			      gboolean        parent_is_visible,
+			      GList          *found_items)
 {
   GooCanvasItemIface *iface = GOO_CANVAS_ITEM_GET_IFACE (item);
 
-  return iface->get_item_at (item, x, y, cr, is_pointer_event,
-			     parent_is_visible);
+  if (iface->get_items_at)
+    return iface->get_items_at (item, x, y, cr, is_pointer_event,
+				parent_is_visible, found_items);
+  else
+    return found_items;
 }
 
 
@@ -1410,7 +1438,10 @@ goo_canvas_item_set_model	  (GooCanvasItem      *item,
  * goo_canvas_item_ensure_updated:
  * @item: a #GooCanvasItem.
  * 
- * Updates the canvas immediately, if an update is scheduled.
+ * This function is only intended to be used when implementing new canvas
+ * items.
+ *
+ * It updates the canvas immediately, if an update is scheduled.
  * This ensures that all item bounds are up-to-date.
  **/
 void
@@ -1431,13 +1462,16 @@ goo_canvas_item_ensure_updated (GooCanvasItem *item)
  * @cr: a cairo context.
  * @bounds: a #GooCanvasBounds to return the new bounds in.
  * 
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically container items such as #GooCanvasGroup.
+ *
  * Updates the item, if needed, and any children.
  **/
 void
-goo_canvas_item_update      (GooCanvasItem      *item,
-			     gboolean            entire_tree,
-			     cairo_t            *cr,
-			     GooCanvasBounds    *bounds)
+goo_canvas_item_update      (GooCanvasItem   *item,
+			     gboolean         entire_tree,
+			     cairo_t         *cr,
+			     GooCanvasBounds *bounds)
 {
   GooCanvasItemIface *iface = GOO_CANVAS_ITEM_GET_IFACE (item);
 
@@ -1453,16 +1487,19 @@ goo_canvas_item_update      (GooCanvasItem      *item,
  * @scale: the scale to use to determine whether an item should be painted.
  *  See #GooCanvasItem:visibility-threshold.
  * 
- * Paints the item and all children if they intersect the given bounds.
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically container items such as #GooCanvasGroup.
+ *
+ * It paints the item and all children if they intersect the given bounds.
  *
  * Note that the @scale argument may be different to the current scale in the
  * #GooCanvasItem, e.g. when the canvas is being printed.
  **/
 void
-goo_canvas_item_paint (GooCanvasItem     *item,
-		       cairo_t           *cr,
-		       GooCanvasBounds   *bounds,
-		       gdouble            scale)
+goo_canvas_item_paint (GooCanvasItem         *item,
+		       cairo_t               *cr,
+		       const GooCanvasBounds *bounds,
+		       gdouble                scale)
 {
   GooCanvasItemIface *iface = GOO_CANVAS_ITEM_GET_IFACE (item);
 
@@ -1477,8 +1514,10 @@ goo_canvas_item_paint (GooCanvasItem     *item,
  * @requested_area: a #GooCanvasBounds to return the requested area in, in the
  *  parent's coordinate space.
  * 
- * This is used by #GooCanvasTable and other layout items to get the requested
- * area of each of their children.
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically layout items such as #GooCanvasTable.
+ *
+ * It gets the requested area of a child item.
  * 
  * Returns: %TRUE if the item should be allocated space.
  **/
@@ -1493,6 +1532,37 @@ goo_canvas_item_get_requested_area (GooCanvasItem    *item,
 }
 
 
+/**
+ * goo_canvas_item_get_requested_height:
+ * @item: a #GooCanvasItem.
+ * @cr: a cairo context.
+ * @width: the width that the item may be allocated.
+ * 
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically layout items such as #GooCanvasTable.
+ *
+ * It gets the requested height of a child item, assuming it is allocated the
+ * given width. This is useful for text items whose requested height may change
+ * depending on the allocated width.
+ * 
+ * Returns: the requested height of the item, given the allocated width,
+ *  or %-1 if the item doesn't support this method or its height doesn't
+ *  change when allocated different widths.
+ **/
+gdouble
+goo_canvas_item_get_requested_height (GooCanvasItem       *item,
+				      cairo_t		  *cr,
+				      gdouble              width)
+{
+  GooCanvasItemIface *iface = GOO_CANVAS_ITEM_GET_IFACE (item);
+
+  if (iface->get_requested_height)
+    return iface->get_requested_height (item, cr, width);
+  else
+    return -1;
+}
+
+
 /**
  * goo_canvas_item_allocate_area:
  * @item: a #GooCanvasItem.
@@ -1506,23 +1576,26 @@ goo_canvas_item_get_requested_area (GooCanvasItem    *item,
  * @y_offset: the y offset of the allocated area from the requested area in
  *  the device coordinate space.
  * 
- * This is used by #GooCanvasTable and other layout items to allocate an area
- * to each of their children.
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically layout items such as #GooCanvasTable.
  *
- * Note that the parent item will use a transform to move each of its children
- * for the layout, so their is no need for the item to reposition itself. It
- * only needs to recalculate its device bounds.
+ * It allocates an area to a child #GooCanvasItem.
  *
- * The @x_offset and @y_offset are provided since for the simple builtin items
- * all they need to do is add these to their bounds.
+ * Note that the parent layout item will use a transform to move each of its
+ * children for the layout, so there is no need for the child item to
+ * reposition itself. It only needs to recalculate its device bounds.
+ *
+ * To help recalculate the item's device bounds, the @x_offset and @y_offset
+ * of the child item's allocated position from its requested position are
+ * provided. Simple items can just add these to their bounds.
  **/
 void
-goo_canvas_item_allocate_area      (GooCanvasItem    *item,
-				    cairo_t          *cr,
-				    GooCanvasBounds  *requested_area,
-				    GooCanvasBounds  *allocated_area,
-				    gdouble           x_offset,
-				    gdouble           y_offset)
+goo_canvas_item_allocate_area      (GooCanvasItem         *item,
+				    cairo_t               *cr,
+				    const GooCanvasBounds *requested_area,
+				    const GooCanvasBounds *allocated_area,
+				    gdouble                x_offset,
+				    gdouble                y_offset)
 {
   GooCanvasItemIface *iface = GOO_CANVAS_ITEM_GET_IFACE (item);
 
@@ -1735,7 +1808,10 @@ _goo_canvas_item_set_child_properties_internal (GObject              *object,
  * @var_args: pairs of property names and value pointers, and a terminating
  *  %NULL.
  * 
- * Gets the values of one or more child properties of @child.
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically layout container items such as #GooCanvasTable.
+ *
+ * It gets the values of one or more child properties of @child.
  **/
 void
 goo_canvas_item_get_child_properties_valist (GooCanvasItem   *item,
@@ -1755,7 +1831,10 @@ goo_canvas_item_get_child_properties_valist (GooCanvasItem   *item,
  * @child: a child #GooCanvasItem.
  * @var_args: pairs of property names and values, and a terminating %NULL.
  * 
- * Sets the values of one or more child properties of @child.
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically layout container items such as #GooCanvasTable.
+ *
+ * It sets the values of one or more child properties of @child.
  **/
 void
 goo_canvas_item_set_child_properties_valist (GooCanvasItem   *item,
@@ -1775,7 +1854,10 @@ goo_canvas_item_set_child_properties_valist (GooCanvasItem   *item,
  * @child: a child #GooCanvasItem.
  * @...: pairs of property names and value pointers, and a terminating %NULL.
  * 
- * Gets the values of one or more child properties of @child.
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically layout container items such as #GooCanvasTable.
+ *
+ * It gets the values of one or more child properties of @child.
  **/
 void
 goo_canvas_item_get_child_properties        (GooCanvasItem   *item,
@@ -1796,7 +1878,10 @@ goo_canvas_item_get_child_properties        (GooCanvasItem   *item,
  * @child: a child #GooCanvasItem.
  * @...: pairs of property names and values, and a terminating %NULL.
  * 
- * Sets the values of one or more child properties of @child.
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically layout container items such as #GooCanvasTable.
+ *
+ * It sets the values of one or more child properties of @child.
  **/
 void
 goo_canvas_item_set_child_properties        (GooCanvasItem   *item,
@@ -1818,7 +1903,10 @@ goo_canvas_item_set_child_properties        (GooCanvasItem   *item,
  * @property_id: the id for the property
  * @pspec: the #GParamSpec for the property
  * 
- * Installs a child property on a canvas item class. 
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically layout container items such as #GooCanvasTable.
+ *
+ * It installs a child property on a canvas item class. 
  **/
 void
 goo_canvas_item_class_install_child_property (GObjectClass *iclass,
@@ -1851,7 +1939,10 @@ goo_canvas_item_class_install_child_property (GObjectClass *iclass,
  * @returns: the #GParamSpec of the child property or %NULL if @class has no
  *   child property with that name.
  *
- * Finds a child property of a canvas item class by name.
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically layout container items such as #GooCanvasTable.
+ *
+ * It finds a child property of a canvas item class by name.
  */
 GParamSpec*
 goo_canvas_item_class_find_child_property (GObjectClass *iclass,
@@ -1872,7 +1963,10 @@ goo_canvas_item_class_find_child_property (GObjectClass *iclass,
  * @returns: a newly allocated array of #GParamSpec*. The array must be 
  *           freed with g_free().
  *
- * Returns all child properties of a canvas item class.
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically layout container items such as #GooCanvasTable.
+ *
+ * It returns all child properties of a canvas item class.
  */
 GParamSpec**
 goo_canvas_item_class_list_child_properties (GObjectClass *iclass,
