@@ -49,6 +49,9 @@ struct _GooCanvasBounds
   gdouble x1, y1, x2, y2;
 };
 
+GType goo_canvas_bounds_get_type (void) G_GNUC_CONST;
+#define GOO_TYPE_CANVAS_BOUNDS (goo_canvas_bounds_get_type ())
+
 
 #define GOO_TYPE_CANVAS_ITEM            (goo_canvas_item_get_type ())
 #define GOO_CANVAS_ITEM(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GOO_TYPE_CANVAS_ITEM, GooCanvasItem))
@@ -91,7 +94,7 @@ typedef struct _GooCanvasItem       GooCanvasItem;
  * @get_parent: gets the item's parent.
  * @set_parent: sets the item's parent.
  * @get_bounds: gets the bounds of the item.
- * @get_item_at: gets the item at the given point.
+ * @get_items_at: gets all the items at the given point.
  * @update: updates the item, if needed. It recalculates the bounds of the item
  *  and requests redraws of parts of the canvas if necessary.
  * @paint: renders the item to the given cairo context.
@@ -107,6 +110,8 @@ typedef struct _GooCanvasItem       GooCanvasItem;
  * @get_style: gets the item's style.
  * @set_style: sets the item's style.
  * @is_visible: returns %TRUE if the item is currently visible.
+ * @get_requested_height: returns the requested height of the item,
+ *  given a particular allocated width, using the parent's coordinate space.
  * @get_model: gets the model that the canvas item is viewing.
  * @set_model: sets the model that the canvas item will view.
  * @enter_notify_event: signal emitted when the mouse enters the item.
@@ -126,7 +131,7 @@ typedef struct _GooCanvasItem       GooCanvasItem;
  * #GooCanvasItem interface.
  *
  * Simple canvas items only need to implement the get_parent(), set_parent(),
- * get_bounds(), get_item_at(), update() and paint() methods (and also
+ * get_bounds(), get_items_at(), update() and paint() methods (and also
  * get_requested_area() and allocate_area() if they are going to be used
  * inside a layout container like #GooCanvasTable).
  *
@@ -187,19 +192,20 @@ struct _GooCanvasItemIface
 							 GooCanvasItem		*parent);
   void			(* get_bounds)			(GooCanvasItem		*item,
 							 GooCanvasBounds	*bounds);
-  GooCanvasItem*	(* get_item_at)			(GooCanvasItem		*item,
+  GList*		(* get_items_at)		(GooCanvasItem		*item,
 							 gdouble		 x,
 							 gdouble		 y,
 							 cairo_t		*cr,
 							 gboolean		 is_pointer_event,
-							 gboolean		 parent_is_visible);
+							 gboolean		 parent_is_visible,
+							 GList                  *found_items);
   void			(* update)			(GooCanvasItem		*item,
 							 gboolean		 entire_tree,
 							 cairo_t		*cr,
 							 GooCanvasBounds	*bounds);
   void			(* paint)			(GooCanvasItem		*item,
 							 cairo_t		*cr,
-							 GooCanvasBounds	*bounds,
+							 const GooCanvasBounds	*bounds,
 							 gdouble		 scale);
 
   gboolean		(* get_requested_area)		(GooCanvasItem		*item,
@@ -207,19 +213,23 @@ struct _GooCanvasItemIface
 							 GooCanvasBounds	*requested_area);
   void			(* allocate_area)		(GooCanvasItem		*item,
 							 cairo_t		*cr,
-							 GooCanvasBounds	*requested_area,
-							 GooCanvasBounds	*allocated_area,
+							 const GooCanvasBounds	*requested_area,
+							 const GooCanvasBounds	*allocated_area,
 							 gdouble		 x_offset,
 							 gdouble		 y_offset);
 
   /* Virtual methods that canvas items may implement. */
-  cairo_matrix_t*	(* get_transform)		(GooCanvasItem		*item);
+  gboolean		(* get_transform)		(GooCanvasItem		*item,
+							 cairo_matrix_t		*transform);
   void			(* set_transform)		(GooCanvasItem		*item,
-							 cairo_matrix_t		*matrix);
+							 const cairo_matrix_t	*transform);
   GooCanvasStyle*	(* get_style)			(GooCanvasItem		*item);
   void			(* set_style)			(GooCanvasItem		*item,
 							 GooCanvasStyle		*style);
   gboolean		(* is_visible)			(GooCanvasItem		*item);
+  gdouble               (* get_requested_height)	(GooCanvasItem		*item,
+							 cairo_t		*cr,
+							 gdouble		 width);
 
   /* Virtual methods that model/view items must implement. */
   GooCanvasItemModel*	(* get_model)			(GooCanvasItem		*item);
@@ -263,11 +273,23 @@ struct _GooCanvasItemIface
 
   /*< private >*/
 
+  /* We might use this in future to support tooltips. */
+  gboolean		(* query_tooltip)		(GooCanvasItem		*item,
+							 gdouble		 x,
+							 gdouble		 y,
+							 gboolean		 keyboard_tooltip,
+							 gpointer /*GtkTooltip*/		*tooltip);
+
+
   /* Padding for future expansion */
   void (*_goo_canvas_reserved1) (void);
   void (*_goo_canvas_reserved2) (void);
   void (*_goo_canvas_reserved3) (void);
   void (*_goo_canvas_reserved4) (void);
+  void (*_goo_canvas_reserved5) (void);
+  void (*_goo_canvas_reserved6) (void);
+  void (*_goo_canvas_reserved7) (void);
+  void (*_goo_canvas_reserved8) (void);
 };
 
 
@@ -318,6 +340,7 @@ void               goo_canvas_item_set_canvas     (GooCanvasItem   *item,
 GooCanvasItem*     goo_canvas_item_get_parent     (GooCanvasItem   *item);
 void               goo_canvas_item_set_parent	  (GooCanvasItem   *item,
 						   GooCanvasItem   *parent);
+void               goo_canvas_item_remove         (GooCanvasItem   *item);
 gboolean           goo_canvas_item_is_container   (GooCanvasItem   *item);
 
 void               goo_canvas_item_raise          (GooCanvasItem   *item,
@@ -325,9 +348,10 @@ void               goo_canvas_item_raise          (GooCanvasItem   *item,
 void               goo_canvas_item_lower          (GooCanvasItem   *item,
 						   GooCanvasItem   *below);
 
-cairo_matrix_t*    goo_canvas_item_get_transform  (GooCanvasItem   *item);
-void               goo_canvas_item_set_transform  (GooCanvasItem   *item,
-						   cairo_matrix_t  *matrix);
+gboolean           goo_canvas_item_get_transform  (GooCanvasItem   *item,
+						   cairo_matrix_t  *transform);
+void               goo_canvas_item_set_transform  (GooCanvasItem         *item,
+						   const cairo_matrix_t  *transform);
 void               goo_canvas_item_set_simple_transform (GooCanvasItem   *item,
 							 gdouble          x,
 							 gdouble          y,
@@ -372,12 +396,13 @@ void               goo_canvas_item_stop_animation (GooCanvasItem   *item);
 
 void               goo_canvas_item_get_bounds	  (GooCanvasItem   *item,
 						   GooCanvasBounds *bounds);
-GooCanvasItem*	   goo_canvas_item_get_item_at    (GooCanvasItem   *item,
+GList*		   goo_canvas_item_get_items_at   (GooCanvasItem   *item,
 						   gdouble          x,
 						   gdouble          y,
 						   cairo_t         *cr,
 						   gboolean         is_pointer_event,
-						   gboolean         parent_is_visible);
+						   gboolean         parent_is_visible,
+						   GList           *found_items);
 gboolean           goo_canvas_item_is_visible     (GooCanvasItem   *item);
 
 GooCanvasItemModel* goo_canvas_item_get_model	  (GooCanvasItem      *item);
@@ -390,20 +415,23 @@ void               goo_canvas_item_update         (GooCanvasItem   *item,
 						   gboolean         entire_tree,
 						   cairo_t         *cr,
 						   GooCanvasBounds *bounds);
-void               goo_canvas_item_paint          (GooCanvasItem   *item,
-						   cairo_t         *cr,
-						   GooCanvasBounds *bounds,
-						   gdouble          scale);
+void               goo_canvas_item_paint          (GooCanvasItem         *item,
+						   cairo_t               *cr,
+						   const GooCanvasBounds *bounds,
+						   gdouble                scale);
 
 gboolean	   goo_canvas_item_get_requested_area (GooCanvasItem	*item,
 						       cairo_t          *cr,
 						       GooCanvasBounds  *requested_area);
-void		   goo_canvas_item_allocate_area      (GooCanvasItem	*item,
-						       cairo_t          *cr,
-						       GooCanvasBounds  *requested_area,
-						       GooCanvasBounds  *allocated_area,
-						       gdouble           x_offset,
-						       gdouble           y_offset);
+gdouble            goo_canvas_item_get_requested_height (GooCanvasItem  *item,
+							 cairo_t	*cr,
+							 gdouble         width);
+void		   goo_canvas_item_allocate_area      (GooCanvasItem	     *item,
+						       cairo_t               *cr,
+						       const GooCanvasBounds *requested_area,
+						       const GooCanvasBounds *allocated_area,
+						       gdouble                x_offset,
+						       gdouble                y_offset);
 
 
 
