@@ -430,6 +430,33 @@ goo_canvas_item_model_set_parent (GooCanvasItemModel *model,
 }
 
 
+/**
+ * goo_canvas_item_model_remove:
+ * @model: an item model.
+ * 
+ * Removes a model from its parent. If the model is in a canvas it will be
+ * removed.
+ *
+ * This would normally also result in the model being freed.
+ **/
+void
+goo_canvas_item_model_remove         (GooCanvasItemModel *model)
+{
+  GooCanvasItemModel *parent;
+  gint child_num;
+
+  parent = goo_canvas_item_model_get_parent (model);
+  if (!parent)
+    return;
+
+  child_num = goo_canvas_item_model_find_child (parent, model);
+  if (child_num == -1)
+    return;
+
+  goo_canvas_item_model_remove_child (parent, child_num);
+}
+
+
 /**
  * goo_canvas_item_model_raise:
  * @model: an item model.
@@ -519,33 +546,35 @@ goo_canvas_item_model_lower          (GooCanvasItemModel *model,
 /**
  * goo_canvas_item_model_get_transform:
  * @model: an item model.
+ * @transform: the place to store the transform.
  * 
  * Gets the transformation matrix of an item model.
  * 
- * Returns: the model's transformation matrix.
+ * Returns: %TRUE if a transform is set.
  **/
-cairo_matrix_t*
-goo_canvas_item_model_get_transform  (GooCanvasItemModel *model)
+gboolean
+goo_canvas_item_model_get_transform  (GooCanvasItemModel *model,
+				      cairo_matrix_t     *transform)
 {
   GooCanvasItemModelIface *iface = GOO_CANVAS_ITEM_MODEL_GET_IFACE (model);
 
-  return iface->get_transform ? iface->get_transform (model) : NULL;
+  return iface->get_transform ? iface->get_transform (model, transform) : FALSE;
 }
 
 
 /**
  * goo_canvas_item_model_set_transform:
  * @model: an item model.
- * @matrix: the new transformation matrix, or %NULL to reset the
+ * @transform: the new transformation matrix, or %NULL to reset the
  *  transformation to the identity matrix.
  * 
  * Sets the transformation matrix of an item model.
  **/
 void
-goo_canvas_item_model_set_transform  (GooCanvasItemModel *model,
-				      cairo_matrix_t     *matrix)
+goo_canvas_item_model_set_transform  (GooCanvasItemModel   *model,
+				      const cairo_matrix_t *transform)
 {
-  GOO_CANVAS_ITEM_MODEL_GET_IFACE (model)->set_transform (model, matrix);
+  GOO_CANVAS_ITEM_MODEL_GET_IFACE (model)->set_transform (model, transform);
 }
 
 
@@ -590,11 +619,9 @@ goo_canvas_item_model_translate      (GooCanvasItemModel *model,
 				      gdouble             ty)
 {
   GooCanvasItemModelIface *iface = GOO_CANVAS_ITEM_MODEL_GET_IFACE (model);
-  cairo_matrix_t *matrix, new_matrix = { 1, 0, 0, 1, 0, 0 };
+  cairo_matrix_t new_matrix = { 1, 0, 0, 1, 0, 0 };
 
-  matrix = iface->get_transform (model);
-  if (matrix)
-    new_matrix = *matrix;
+  iface->get_transform (model, &new_matrix);
   cairo_matrix_translate (&new_matrix, tx, ty);
   iface->set_transform (model, &new_matrix);
 }
@@ -614,11 +641,9 @@ goo_canvas_item_model_scale          (GooCanvasItemModel *model,
 				      gdouble             sy)
 {
   GooCanvasItemModelIface *iface = GOO_CANVAS_ITEM_MODEL_GET_IFACE (model);
-  cairo_matrix_t *matrix, new_matrix = { 1, 0, 0, 1, 0, 0 };
+  cairo_matrix_t new_matrix = { 1, 0, 0, 1, 0, 0 };
 
-  matrix = iface->get_transform (model);
-  if (matrix)
-    new_matrix = *matrix;
+  iface->get_transform (model, &new_matrix);
   cairo_matrix_scale (&new_matrix, sx, sy);
   iface->set_transform (model, &new_matrix);
 }
@@ -641,12 +666,10 @@ goo_canvas_item_model_rotate         (GooCanvasItemModel *model,
 				      gdouble             cy)
 {
   GooCanvasItemModelIface *iface = GOO_CANVAS_ITEM_MODEL_GET_IFACE (model);
-  cairo_matrix_t *matrix, new_matrix = { 1, 0, 0, 1, 0, 0 };
+  cairo_matrix_t new_matrix = { 1, 0, 0, 1, 0, 0 };
   double radians = degrees * (M_PI / 180);
 
-  matrix = iface->get_transform (model);
-  if (matrix)
-    new_matrix = *matrix;
+  iface->get_transform (model, &new_matrix);
   cairo_matrix_translate (&new_matrix, cx, cy);
   cairo_matrix_rotate (&new_matrix, radians);
   cairo_matrix_translate (&new_matrix, -cx, -cy);
@@ -671,12 +694,10 @@ goo_canvas_item_model_skew_x         (GooCanvasItemModel *model,
 				      gdouble             cy)
 {
   GooCanvasItemModelIface *iface = GOO_CANVAS_ITEM_MODEL_GET_IFACE (model);
-  cairo_matrix_t *matrix, tmp, new_matrix = { 1, 0, 0, 1, 0, 0 };
+  cairo_matrix_t tmp, new_matrix = { 1, 0, 0, 1, 0, 0 };
   double radians = degrees * (M_PI / 180);
 
-  matrix = iface->get_transform (model);
-  if (matrix)
-    new_matrix = *matrix;
+  iface->get_transform (model, &new_matrix);
   cairo_matrix_translate (&new_matrix, cx, cy);
   cairo_matrix_init (&tmp, 1, 0, tan (radians), 1, 0, 0);
   cairo_matrix_multiply (&new_matrix, &tmp, &new_matrix);
@@ -702,12 +723,10 @@ goo_canvas_item_model_skew_y         (GooCanvasItemModel *model,
 				      gdouble             cy)
 {
   GooCanvasItemModelIface *iface = GOO_CANVAS_ITEM_MODEL_GET_IFACE (model);
-  cairo_matrix_t *matrix, tmp, new_matrix = { 1, 0, 0, 1, 0, 0 };
+  cairo_matrix_t tmp, new_matrix = { 1, 0, 0, 1, 0, 0 };
   double radians = degrees * (M_PI / 180);
 
-  matrix = iface->get_transform (model);
-  if (matrix)
-    new_matrix = *matrix;
+  iface->get_transform (model, &new_matrix);
   cairo_matrix_translate (&new_matrix, cx, cy);
   cairo_matrix_init (&tmp, 1, tan (radians), 0, 1, 0, 0);
   cairo_matrix_multiply (&new_matrix, &tmp, &new_matrix);
@@ -830,7 +849,11 @@ extern void _goo_canvas_item_set_child_properties_internal (GObject *object, GOb
  * @var_args: pairs of property names and value pointers, and a terminating
  *  %NULL.
  * 
- * Gets the values of one or more child properties of @child.
+ * This function is only intended to be used when implementing new canvas
+ * item models, specifically layout container item models such as
+ * #GooCanvasTableModel.
+ *
+ * It gets the values of one or more child properties of @child.
  **/
 void
 goo_canvas_item_model_get_child_properties_valist (GooCanvasItemModel *model,
@@ -850,7 +873,11 @@ goo_canvas_item_model_get_child_properties_valist (GooCanvasItemModel *model,
  * @child: a child #GooCanvasItemModel.
  * @var_args: pairs of property names and values, and a terminating %NULL.
  * 
- * Sets the values of one or more child properties of @child.
+ * This function is only intended to be used when implementing new canvas
+ * item models, specifically layout container item models such as
+ * #GooCanvasTableModel.
+ *
+ * It sets the values of one or more child properties of @child.
  **/
 void
 goo_canvas_item_model_set_child_properties_valist (GooCanvasItemModel *model,
@@ -870,7 +897,11 @@ goo_canvas_item_model_set_child_properties_valist (GooCanvasItemModel *model,
  * @child: a child #GooCanvasItemModel.
  * @...: pairs of property names and value pointers, and a terminating %NULL.
  * 
- * Gets the values of one or more child properties of @child.
+ * This function is only intended to be used when implementing new canvas
+ * item models, specifically layout container item models such as
+ * #GooCanvasTableModel.
+ *
+ * It gets the values of one or more child properties of @child.
  **/
 void
 goo_canvas_item_model_get_child_properties  (GooCanvasItemModel   *model,
@@ -891,7 +922,11 @@ goo_canvas_item_model_get_child_properties  (GooCanvasItemModel   *model,
  * @child: a child #GooCanvasItemModel.
  * @...: pairs of property names and values, and a terminating %NULL.
  * 
- * Sets the values of one or more child properties of @child.
+ * This function is only intended to be used when implementing new canvas
+ * item models, specifically layout container item models such as
+ * #GooCanvasTableModel.
+ *
+ * It sets the values of one or more child properties of @child.
  **/
 void
 goo_canvas_item_model_set_child_properties  (GooCanvasItemModel   *model,
@@ -913,7 +948,11 @@ goo_canvas_item_model_set_child_properties  (GooCanvasItemModel   *model,
  * @property_id: the id for the property
  * @pspec: the #GParamSpec for the property
  * 
- * Installs a child property on a canvas item class. 
+ * This function is only intended to be used when implementing new canvas
+ * item models, specifically layout container item models such as
+ * #GooCanvasTableModel.
+ *
+ * It installs a child property on a canvas item class. 
  **/
 void
 goo_canvas_item_model_class_install_child_property (GObjectClass *mclass,
@@ -946,7 +985,11 @@ goo_canvas_item_model_class_install_child_property (GObjectClass *mclass,
  * @returns: the #GParamSpec of the child property or %NULL if @class has no
  *   child property with that name.
  *
- * Finds a child property of a canvas item class by name.
+ * This function is only intended to be used when implementing new canvas
+ * item models, specifically layout container item models such as
+ * #GooCanvasTableModel.
+ *
+ * It finds a child property of a canvas item class by name.
  */
 GParamSpec*
 goo_canvas_item_model_class_find_child_property (GObjectClass *mclass,
@@ -967,7 +1010,11 @@ goo_canvas_item_model_class_find_child_property (GObjectClass *mclass,
  * @returns: a newly allocated array of #GParamSpec*. The array must be 
  *           freed with g_free().
  *
- * Returns all child properties of a canvas item class.
+ * This function is only intended to be used when implementing new canvas
+ * item models, specifically layout container item models such as
+ * #GooCanvasTableModel.
+ *
+ * It returns all child properties of a canvas item class.
  */
 GParamSpec**
 goo_canvas_item_model_class_list_child_properties (GObjectClass *mclass,
@@ -985,3 +1032,19 @@ goo_canvas_item_model_class_list_child_properties (GObjectClass *mclass,
 
   return pspecs;
 }
+
+
+void
+_goo_canvas_item_model_emit_child_added (GooCanvasItemModel *model,
+					 gint                position)
+{
+  g_signal_emit (model, item_model_signals[CHILD_ADDED], 0, position);
+}
+
+
+void
+_goo_canvas_item_model_emit_changed (GooCanvasItemModel *model,
+				     gboolean            recompute_bounds)
+{
+  g_signal_emit (model, item_model_signals[CHANGED], 0, recompute_bounds);
+}
