@@ -22,7 +22,7 @@
  * method. (#GooCanvasEllipse, #GooCanvasRect and #GooCanvasPath do this.)
  *
  * More complicated items need to implement the update(), paint() and
- * get_item_at() methods. (#GooCanvasImage, #GooCanvasPolyline,
+ * is_item_at() methods. (#GooCanvasImage, #GooCanvasPolyline,
  * #GooCanvasText and #GooCanvasWidget do this.) They may also need to
  * override some of the other GooCanvasItem methods such as set_canvas(),
  * set_parent() or allocate_area() if special code is needed. (#GooCanvasWidget
@@ -58,6 +58,7 @@ enum {
   /* Font properties. */
   PROP_FONT,
   PROP_FONT_DESC,
+  PROP_HINT_METRICS,
 
   /* Convenience properties. */
   PROP_STROKE_COLOR,
@@ -81,9 +82,11 @@ enum {
   PROP_CLIP_FILL_RULE
 };
 
+static gboolean accessibility_enabled = FALSE;
 
-static void canvas_item_interface_init  (GooCanvasItemIface *iface);
-static void goo_canvas_item_simple_finalize (GObject *object);
+static void canvas_item_interface_init          (GooCanvasItemIface   *iface);
+static void goo_canvas_item_simple_dispose      (GObject              *object);
+static void goo_canvas_item_simple_finalize     (GObject              *object);
 static void goo_canvas_item_simple_get_property (GObject              *object,
 						 guint                 prop_id,
 						 GValue               *value,
@@ -106,14 +109,14 @@ goo_canvas_item_simple_install_common_properties (GObjectClass *gobject_class)
   g_object_class_install_property (gobject_class, PROP_STROKE_PATTERN,
                                    g_param_spec_boxed ("stroke-pattern",
 						       _("Stroke Pattern"),
-						       _("The pattern to use to paint the perimeter of the item"),
+						       _("The pattern to use to paint the perimeter of the item, or NULL disable painting"),
 						       GOO_TYPE_CAIRO_PATTERN,
 						       G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_FILL_PATTERN,
                                    g_param_spec_boxed ("fill-pattern",
 						       _("Fill Pattern"),
-						       _("The pattern to use to paint the interior of the item"),
+						       _("The pattern to use to paint the interior of the item, or NULL to disable painting"),
 						       GOO_TYPE_CAIRO_PATTERN,
 						       G_PARAM_READWRITE));
 
@@ -138,7 +141,7 @@ goo_canvas_item_simple_install_common_properties (GObjectClass *gobject_class)
 						      _("Antialias"),
 						      _("The antialiasing mode to use"),
 						      GOO_TYPE_CAIRO_ANTIALIAS,
-						      CAIRO_ANTIALIAS_DEFAULT,
+						      CAIRO_ANTIALIAS_GRAY,
 						      G_PARAM_READWRITE));
 
   /* Line style & width properties. */
@@ -146,7 +149,7 @@ goo_canvas_item_simple_install_common_properties (GObjectClass *gobject_class)
 				   g_param_spec_double ("line-width",
 							_("Line Width"),
 							_("The line width to use for the item's perimeter"),
-							0.0, G_MAXDOUBLE, 0.0,
+							0.0, G_MAXDOUBLE, 2.0,
 							G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_LINE_CAP,
@@ -194,46 +197,54 @@ goo_canvas_item_simple_install_common_properties (GObjectClass *gobject_class)
 						       PANGO_TYPE_FONT_DESCRIPTION,
 						       G_PARAM_READWRITE));
 
+  g_object_class_install_property (gobject_class, PROP_HINT_METRICS,
+				   g_param_spec_enum ("hint-metrics",
+						      _("Hint Metrics"),
+						      _("The hinting to be used for font metrics"),
+						      GOO_TYPE_CAIRO_HINT_METRICS,
+						      CAIRO_HINT_METRICS_OFF,
+						      G_PARAM_READWRITE));
+
   /* Convenience properties - writable only. */
   g_object_class_install_property (gobject_class, PROP_STROKE_COLOR,
 				   g_param_spec_string ("stroke-color",
 							_("Stroke Color"),
-							_("The color to use for the item's perimeter"),
+							_("The color to use for the item's perimeter. To disable painting set the 'stroke-pattern' property to NULL"),
 							NULL,
 							G_PARAM_WRITABLE));
 
   g_object_class_install_property (gobject_class, PROP_STROKE_COLOR_RGBA,
 				   g_param_spec_uint ("stroke-color-rgba",
 						      _("Stroke Color RGBA"),
-						      _("The color to use for the item's perimeter, specified as a 32-bit integer value"),
+						      _("The color to use for the item's perimeter, specified as a 32-bit integer value. To disable painting set the 'stroke-pattern' property to NULL"),
 						      0, G_MAXUINT, 0,
 						      G_PARAM_WRITABLE));
 
   g_object_class_install_property (gobject_class, PROP_STROKE_PIXBUF,
                                    g_param_spec_object ("stroke-pixbuf",
 							_("Stroke Pixbuf"),
-							_("The pixbuf to use to draw the item's perimeter"),
+							_("The pixbuf to use to draw the item's perimeter. To disable painting set the 'stroke-pattern' property to NULL"),
                                                         GDK_TYPE_PIXBUF,
                                                         G_PARAM_WRITABLE));
 
   g_object_class_install_property (gobject_class, PROP_FILL_COLOR,
 				   g_param_spec_string ("fill-color",
 							_("Fill Color"),
-							_("The color to use to paint the interior of the item"),
+							_("The color to use to paint the interior of the item. To disable painting set the 'fill-pattern' property to NULL"),
 							NULL,
 							G_PARAM_WRITABLE));
 
   g_object_class_install_property (gobject_class, PROP_FILL_COLOR_RGBA,
 				   g_param_spec_uint ("fill-color-rgba",
 						      _("Fill Color RGBA"),
-						      _("The color to use to paint the interior of the item, specified as a 32-bit integer value"),
+						      _("The color to use to paint the interior of the item, specified as a 32-bit integer value. To disable painting set the 'fill-pattern' property to NULL"),
 						      0, G_MAXUINT, 0,
 						      G_PARAM_WRITABLE));
 
   g_object_class_install_property (gobject_class, PROP_FILL_PIXBUF,
                                    g_param_spec_object ("fill-pixbuf",
 							_("Fill Pixbuf"),
-							_("The pixbuf to use to paint the interior of the item"),
+							_("The pixbuf to use to paint the interior of the item. To disable painting set the 'fill-pattern' property to NULL"),
                                                         GDK_TYPE_PIXBUF,
                                                         G_PARAM_WRITABLE));
 
@@ -301,6 +312,7 @@ goo_canvas_item_simple_class_init (GooCanvasItemSimpleClass *klass)
 {
   GObjectClass *gobject_class = (GObjectClass*) klass;
 
+  gobject_class->dispose  = goo_canvas_item_simple_dispose;
   gobject_class->finalize = goo_canvas_item_simple_finalize;
 
   gobject_class->get_property = goo_canvas_item_simple_get_property;
@@ -309,6 +321,7 @@ goo_canvas_item_simple_class_init (GooCanvasItemSimpleClass *klass)
   /* Register our accessible factory, but only if accessibility is enabled. */
   if (!ATK_IS_NO_OP_OBJECT_FACTORY (atk_registry_get_factory (atk_get_default_registry (), GTK_TYPE_WIDGET)))
     {
+      accessibility_enabled = TRUE;
       atk_registry_set_factory_type (atk_get_default_registry (),
 				     GOO_TYPE_CANVAS_ITEM_SIMPLE,
 				     goo_canvas_item_accessible_factory_get_type ());
@@ -366,12 +379,13 @@ goo_canvas_item_simple_free_data (GooCanvasItemSimpleData *simple_data)
 	}
 
       g_slice_free (cairo_matrix_t, simple_data->transform);
+      simple_data->transform = NULL;
     }
 }
 
 
 static void
-goo_canvas_item_simple_finalize (GObject *object)
+goo_canvas_item_simple_dispose (GObject *object)
 {
   GooCanvasItemSimple *simple = (GooCanvasItemSimple*) object;
 
@@ -382,6 +396,16 @@ goo_canvas_item_simple_finalize (GObject *object)
 
   goo_canvas_item_simple_reset_model (simple);
   goo_canvas_item_simple_free_data (simple->simple_data);
+
+  G_OBJECT_CLASS (goo_canvas_item_simple_parent_class)->dispose (object);
+}
+
+
+static void
+goo_canvas_item_simple_finalize (GObject *object)
+{
+  GooCanvasItemSimple *simple = (GooCanvasItemSimple*) object;
+
   g_slice_free (GooCanvasItemSimpleData, simple->simple_data);
   simple->simple_data = NULL;
 
@@ -392,12 +416,14 @@ goo_canvas_item_simple_finalize (GObject *object)
 static void
 goo_canvas_item_simple_get_common_property (GObject                 *object,
 					    GooCanvasItemSimpleData *simple_data,
+					    GooCanvas               *canvas,
 					    guint                    prop_id,
 					    GValue                  *value,
 					    GParamSpec              *pspec)
 {
   GooCanvasStyle *style = simple_data->style;
   GValue *svalue;
+  gdouble line_width = 2.0;
   gchar *font = NULL;
 
   switch (prop_id)
@@ -421,13 +447,17 @@ goo_canvas_item_simple_get_common_property (GObject                 *object,
       break;
     case PROP_ANTIALIAS:
       svalue = goo_canvas_style_get_property (style, goo_canvas_style_antialias_id);
-      g_value_set_enum (value, svalue ? svalue->data[0].v_long : CAIRO_ANTIALIAS_DEFAULT);
+      g_value_set_enum (value, svalue ? svalue->data[0].v_long : CAIRO_ANTIALIAS_GRAY);
       break;
 
       /* Line style & width properties. */
     case PROP_LINE_WIDTH:
       svalue = goo_canvas_style_get_property (style, goo_canvas_style_line_width_id);
-      g_value_set_double (value, svalue ? svalue->data[0].v_double : 2.0);
+      if (svalue)
+	line_width = svalue->data[0].v_double;
+      else if (canvas)
+	line_width = goo_canvas_get_default_line_width (canvas);
+      g_value_set_double (value, line_width);
       break;
     case PROP_LINE_CAP:
       svalue = goo_canvas_style_get_property (style, goo_canvas_style_line_cap_id);
@@ -458,6 +488,10 @@ goo_canvas_item_simple_get_common_property (GObject                 *object,
       svalue = goo_canvas_style_get_property (style, goo_canvas_style_font_desc_id);
       g_value_set_boxed (value, svalue ? svalue->data[0].v_pointer : NULL);
       break;
+    case PROP_HINT_METRICS:
+      svalue = goo_canvas_style_get_property (style, goo_canvas_style_hint_metrics_id);
+      g_value_set_enum (value, svalue ? svalue->data[0].v_long : CAIRO_HINT_METRICS_OFF);
+      break;
 
       /* Other properties. */
     case PROP_TRANSFORM:
@@ -509,7 +543,8 @@ goo_canvas_item_simple_get_property (GObject              *object,
       g_value_set_string (value, atk_object_get_description (accessible));
       break;
     default:
-      goo_canvas_item_simple_get_common_property (object, simple_data, prop_id,
+      goo_canvas_item_simple_get_common_property (object, simple_data,
+						  simple->canvas, prop_id,
 						  value, pspec);
       break;
     }
@@ -609,6 +644,10 @@ goo_canvas_item_simple_set_common_property (GObject                 *object,
       goo_canvas_style_set_property (style, goo_canvas_style_font_desc_id, value);
       recompute_bounds = TRUE;
       break;
+    case PROP_HINT_METRICS:
+      goo_canvas_style_set_property (style, goo_canvas_style_hint_metrics_id, value);
+      recompute_bounds = TRUE;
+      break;
 
       /* Convenience properties. */
     case PROP_STROKE_COLOR:
@@ -801,8 +840,11 @@ goo_canvas_item_simple_set_parent (GooCanvasItem  *item,
 				   GooCanvasItem  *parent)
 {
   GooCanvasItemSimple *simple = (GooCanvasItemSimple*) item;
+  GooCanvas *canvas;
+
   simple->parent = parent;
-  simple->canvas = parent ? goo_canvas_item_get_canvas (parent) : NULL;
+  canvas = parent ? goo_canvas_item_get_canvas (parent) : NULL;
+  goo_canvas_item_set_canvas (item, canvas);
   simple->need_update = TRUE;
   simple->need_entire_subtree_update = TRUE;
 }
@@ -842,16 +884,24 @@ goo_canvas_item_simple_changed    (GooCanvasItemSimple *item,
 }
 
 
-static cairo_matrix_t*
-goo_canvas_item_simple_get_transform (GooCanvasItem       *item)
+static gboolean
+goo_canvas_item_simple_get_transform (GooCanvasItem       *item,
+				      cairo_matrix_t      *matrix)
 {
-  return GOO_CANVAS_ITEM_SIMPLE (item)->simple_data->transform;
+  GooCanvasItemSimple *simple = (GooCanvasItemSimple*) item;
+  GooCanvasItemSimpleData *simple_data = simple->simple_data;
+
+  if (!simple_data->transform)
+    return FALSE;
+
+  *matrix = *simple_data->transform;
+  return TRUE;
 }
 
 
 static void
-goo_canvas_item_simple_set_transform (GooCanvasItem  *item,
-				      cairo_matrix_t *transform)
+goo_canvas_item_simple_set_transform (GooCanvasItem        *item,
+				      const cairo_matrix_t *transform)
 {
   GooCanvasItemSimple *simple = (GooCanvasItemSimple*) item;
   GooCanvasItemSimpleData *simple_data = simple->simple_data;
@@ -920,21 +970,22 @@ goo_canvas_item_simple_get_bounds  (GooCanvasItem   *item,
 }
 
 
-static GooCanvasItem*
-goo_canvas_item_simple_get_item_at (GooCanvasItem  *item,
-				    gdouble         x,
-				    gdouble         y,
-				    cairo_t        *cr,
-				    gboolean        is_pointer_event,
-				    gboolean        parent_visible)
+static GList*
+goo_canvas_item_simple_get_items_at (GooCanvasItem  *item,
+				     gdouble         x,
+				     gdouble         y,
+				     cairo_t        *cr,
+				     gboolean        is_pointer_event,
+				     gboolean        parent_visible,
+				     GList          *found_items)
 {
   GooCanvasItemSimpleClass *class = GOO_CANVAS_ITEM_SIMPLE_GET_CLASS (item);
   GooCanvasItemSimple *simple = (GooCanvasItemSimple*) item;
   GooCanvasItemSimpleData *simple_data = simple->simple_data;
-  GooCanvasItem *found = NULL;
   double user_x = x, user_y = y;
   GooCanvasPointerEvents pointer_events = GOO_CANVAS_EVENTS_ALL;
   cairo_matrix_t matrix;
+  gboolean add_item = FALSE;
 
   if (simple->need_update)
     goo_canvas_item_ensure_updated (item);
@@ -942,19 +993,19 @@ goo_canvas_item_simple_get_item_at (GooCanvasItem  *item,
   /* Skip the item if the point isn't in the item's bounds. */
   if (simple->bounds.x1 > x || simple->bounds.x2 < x
       || simple->bounds.y1 > y || simple->bounds.y2 < y)
-    return NULL;
+    return found_items;
 
   /* Check if the item should receive events. */
   if (is_pointer_event)
     {
       if (simple_data->pointer_events == GOO_CANVAS_EVENTS_NONE)
-	return NULL;
+	return found_items;
       if (simple_data->pointer_events & GOO_CANVAS_EVENTS_VISIBLE_MASK
 	  && (!parent_visible
 	      || simple_data->visibility <= GOO_CANVAS_ITEM_INVISIBLE
 	      || (simple_data->visibility == GOO_CANVAS_ITEM_VISIBLE_ABOVE_THRESHOLD
 		  && simple->canvas->scale < simple_data->visibility_threshold)))
-	return NULL;
+	return found_items;
 
       pointer_events = simple_data->pointer_events;
     }
@@ -970,6 +1021,7 @@ goo_canvas_item_simple_get_item_at (GooCanvasItem  *item,
   matrix.x0 = matrix.y0 = 0.0;
   cairo_set_matrix (cr, &matrix);
 
+  /* If the item has a clip path, check if the point is inside it. */
   if (simple_data->clip_path_commands)
     {
       goo_canvas_create_path (simple_data->clip_path_commands, cr);
@@ -977,14 +1029,14 @@ goo_canvas_item_simple_get_item_at (GooCanvasItem  *item,
       if (!cairo_in_fill (cr, user_x, user_y))
 	{
 	  cairo_restore (cr);
-	  return NULL;
+	  return found_items;
 	}
     }
 
-  if (class->simple_get_item_at)
+  if (class->simple_is_item_at)
     {
-      found = class->simple_get_item_at (simple, user_x, user_y, cr,
-					 is_pointer_event);
+      add_item = class->simple_is_item_at (simple, user_x, user_y, cr,
+					   is_pointer_event);
     }
   else
     {
@@ -993,12 +1045,15 @@ goo_canvas_item_simple_get_item_at (GooCanvasItem  *item,
 
       if (goo_canvas_item_simple_check_in_path (simple, user_x, user_y, cr,
 						pointer_events))
-	found = item;
+	add_item = TRUE;
     }
 
   cairo_restore (cr);
 
-  return found;
+  if (add_item)
+    return g_list_prepend (found_items, item);
+  else
+    return found_items;
 }
 
 
@@ -1227,12 +1282,12 @@ goo_canvas_item_simple_get_requested_area (GooCanvasItem    *item,
 
 
 static void
-goo_canvas_item_simple_allocate_area      (GooCanvasItem    *item,
-					   cairo_t          *cr,
-					   GooCanvasBounds  *requested_area,
-					   GooCanvasBounds  *allocated_area,
-					   gdouble           x_offset,
-					   gdouble           y_offset)
+goo_canvas_item_simple_allocate_area      (GooCanvasItem         *item,
+					   cairo_t               *cr,
+					   const GooCanvasBounds *requested_area,
+					   const GooCanvasBounds *allocated_area,
+					   gdouble                x_offset,
+					   gdouble                y_offset)
 {
   GooCanvasItemSimple *simple = (GooCanvasItemSimple*) item;
 
@@ -1246,10 +1301,10 @@ goo_canvas_item_simple_allocate_area      (GooCanvasItem    *item,
 
 
 static void
-goo_canvas_item_simple_paint (GooCanvasItem     *item,
-			      cairo_t           *cr,
-			      GooCanvasBounds   *bounds,
-			      gdouble            scale)
+goo_canvas_item_simple_paint (GooCanvasItem         *item,
+			      cairo_t               *cr,
+			      const GooCanvasBounds *bounds,
+			      gdouble                scale)
 {
   GooCanvasItemSimpleClass *class = GOO_CANVAS_ITEM_SIMPLE_GET_CLASS (item);
   GooCanvasItemSimple *simple = (GooCanvasItemSimple*) item;
@@ -1270,6 +1325,7 @@ goo_canvas_item_simple_paint (GooCanvasItem     *item,
   if (simple_data->transform)
     cairo_transform (cr, simple_data->transform);
 
+  /* Clip with the item's clip path, if it is set. */
   if (simple_data->clip_path_commands)
     {
       goo_canvas_create_path (simple_data->clip_path_commands, cr);
@@ -1379,7 +1435,8 @@ goo_canvas_item_simple_set_model (GooCanvasItemSimple  *item,
   item->model = g_object_ref (model);
   item->simple_data = &item->model->simple_data;
 
-  goo_canvas_item_simple_setup_accessibility (item);
+  if (accessibility_enabled)
+    goo_canvas_item_simple_setup_accessibility (item);
 
   g_signal_connect (model, "changed",
 		    G_CALLBACK (goo_canvas_item_model_simple_changed),
@@ -1404,7 +1461,7 @@ canvas_item_interface_init (GooCanvasItemIface *iface)
   iface->get_parent	    = goo_canvas_item_simple_get_parent;
   iface->set_parent	    = goo_canvas_item_simple_set_parent;
   iface->get_bounds         = goo_canvas_item_simple_get_bounds;
-  iface->get_item_at	    = goo_canvas_item_simple_get_item_at;
+  iface->get_items_at	    = goo_canvas_item_simple_get_items_at;
   iface->update             = goo_canvas_item_simple_update;
   iface->get_requested_area = goo_canvas_item_simple_get_requested_area;
   iface->allocate_area      = goo_canvas_item_simple_allocate_area;
@@ -1426,7 +1483,9 @@ canvas_item_interface_init (GooCanvasItemIface *iface)
  * @item: a #GooCanvasItemSimple.
  * @cr: a cairo context.
  * 
- * Paints the current path, using the item's style settings.
+ * This function is intended to be used by subclasses of #GooCanvasItemSimple.
+ *
+ * It paints the current path, using the item's style settings.
  **/
 void
 goo_canvas_item_simple_paint_path (GooCanvasItemSimple *item,
@@ -1454,13 +1513,16 @@ goo_canvas_item_simple_paint_path (GooCanvasItemSimple *item,
  * @cr: a cairo context.
  * @bounds: the #GooCanvasBounds struct to store the resulting bounding box.
  * 
- * Calculates the bounds of the current path, storing the results in the given
- * #GooCanvasBounds struct.
+ * This function is intended to be used by subclasses of #GooCanvasItemSimple,
+ * typically in their update() or get_requested_area() methods.
  *
- * The returned bounds contains the bounding box of the item in device space,
+ * It calculates the bounds of the current path, using the item's style
+ * settings, and stores the results in the given #GooCanvasBounds struct.
+ *
+ * The returned bounds contains the bounding box of the path in device space,
  * converted to user space coordinates. To calculate the bounds completely in
  * user space, use cairo_identity_matrix() to temporarily reset the current
- * transformation matrix.
+ * transformation matrix to the identity matrix.
  **/
 void
 goo_canvas_item_simple_get_path_bounds (GooCanvasItemSimple *item,
@@ -1468,37 +1530,66 @@ goo_canvas_item_simple_get_path_bounds (GooCanvasItemSimple *item,
 					GooCanvasBounds     *bounds)
 {
   GooCanvasStyle *style = item->simple_data->style;
-  GooCanvasBounds tmp_bounds, tmp_bounds2;
+  GooCanvasBounds fill_bounds, stroke_bounds;
 
   /* Calculate the filled extents. */
   goo_canvas_style_set_fill_options (style, cr);
-  cairo_fill_extents (cr, &tmp_bounds.x1, &tmp_bounds.y1,
-		      &tmp_bounds.x2, &tmp_bounds.y2);
+  cairo_fill_extents (cr, &fill_bounds.x1, &fill_bounds.y1,
+		      &fill_bounds.x2, &fill_bounds.y2);
 
   /* Check the stroke. */
   goo_canvas_style_set_stroke_options (style, cr);
-  cairo_stroke_extents (cr, &tmp_bounds2.x1, &tmp_bounds2.y1,
-			&tmp_bounds2.x2, &tmp_bounds2.y2);
+  cairo_stroke_extents (cr, &stroke_bounds.x1, &stroke_bounds.y1,
+			&stroke_bounds.x2, &stroke_bounds.y2);
 
-  /* FIXME: Handle whatever cairo returns for NULL bounds. Cairo starts with
-     INT16_MAX << 16 in cairo-traps.c, but this may get converted to user
-     space so could be anything? */
+  /* Workaround for cairo < 1.4.0. It used to just return odd values
+     if the path had empty bounds. This fix will work, but only if there is
+     no transform currently set, since cairo will convert to user space. */
+  if (cairo_version () < CAIRO_VERSION_ENCODE (1, 4, 0))
+    {
+      if (fill_bounds.x1 == 32767.0 && fill_bounds.x2 == -32768.0)
+	fill_bounds.x1 = fill_bounds.x2 = 0.0;
+      if (stroke_bounds.x1 == 32767.0 && stroke_bounds.x2 == -32768.0)
+	stroke_bounds.x1 = stroke_bounds.x2 = 0.0;
+    }
 
-  bounds->x1 = MIN (tmp_bounds.x1, tmp_bounds.x2);
-  bounds->x1 = MIN (bounds->x1, tmp_bounds2.x1);
-  bounds->x1 = MIN (bounds->x1, tmp_bounds2.x2);
+  if (fill_bounds.x1 == 0.0 && fill_bounds.x2 == 0.0)
+    {
+      /* The fill bounds are empty so just use the stroke bounds.
+	 If the stroke bounds are also empty the bounds will be all 0.0. */
+      bounds->x1 = MIN (stroke_bounds.x1, stroke_bounds.x2);
+      bounds->x2 = MAX (stroke_bounds.x1, stroke_bounds.x2);
+      bounds->y1 = MIN (stroke_bounds.y1, stroke_bounds.y2);
+      bounds->y2 = MAX (stroke_bounds.y1, stroke_bounds.y2);
+    }
+  else if (stroke_bounds.x1 == 0.0 && stroke_bounds.x2 == 0.0)
+    {
+      /* The stroke bounds are empty so just use the fill bounds. */
+      bounds->x1 = MIN (fill_bounds.x1, fill_bounds.x2);
+      bounds->x2 = MAX (fill_bounds.x1, fill_bounds.x2);
+      bounds->y1 = MIN (fill_bounds.y1, fill_bounds.y2);
+      bounds->y2 = MAX (fill_bounds.y1, fill_bounds.y2);
+    }
+  else
+    {
+      /* Both fill & stoke bounds are non-empty so combine them. */
+      bounds->x1 = MIN (fill_bounds.x1, fill_bounds.x2);
+      bounds->x2 = MAX (fill_bounds.x1, fill_bounds.x2);
+      bounds->y1 = MIN (fill_bounds.y1, fill_bounds.y2);
+      bounds->y2 = MAX (fill_bounds.y1, fill_bounds.y2);
 
-  bounds->x2 = MAX (tmp_bounds.x1, tmp_bounds.x2);
-  bounds->x2 = MAX (bounds->x2, tmp_bounds2.x1);
-  bounds->x2 = MAX (bounds->x2, tmp_bounds2.x2);
+      bounds->x1 = MIN (bounds->x1, stroke_bounds.x1);
+      bounds->x1 = MIN (bounds->x1, stroke_bounds.x2);
 
-  bounds->y1 = MIN (tmp_bounds.y1, tmp_bounds.y2);
-  bounds->y1 = MIN (bounds->y1, tmp_bounds2.y1);
-  bounds->y1 = MIN (bounds->y1, tmp_bounds2.y2);
+      bounds->x2 = MAX (bounds->x2, stroke_bounds.x1);
+      bounds->x2 = MAX (bounds->x2, stroke_bounds.x2);
 
-  bounds->y2 = MAX (tmp_bounds.y1, tmp_bounds.y2);
-  bounds->y2 = MAX (bounds->y2, tmp_bounds2.y1);
-  bounds->y2 = MAX (bounds->y2, tmp_bounds2.y2);
+      bounds->y1 = MIN (bounds->y1, stroke_bounds.y1);
+      bounds->y1 = MIN (bounds->y1, stroke_bounds.y2);
+
+      bounds->y2 = MAX (bounds->y2, stroke_bounds.y1);
+      bounds->y2 = MAX (bounds->y2, stroke_bounds.y2);
+    }
 }
 
 
@@ -1508,7 +1599,10 @@ goo_canvas_item_simple_get_path_bounds (GooCanvasItemSimple *item,
  * @cr: a cairo context.
  * @bounds: the bounds of the item, in the item's coordinate space.
  * 
- * Converts the item's bounds to a bounding box in device space.
+ * This function is intended to be used by subclasses of #GooCanvasItemSimple,
+ * typically in their update() or get_requested_area() methods.
+ *
+ * It converts the item's bounds to a bounding box in device space.
  **/
 void
 goo_canvas_item_simple_user_bounds_to_device (GooCanvasItemSimple *item,
@@ -1553,12 +1647,12 @@ goo_canvas_item_simple_user_bounds_to_device (GooCanvasItemSimple *item,
  * @cr: a cairo context.
  * @bounds: the bounds of the item, in the item's coordinate space.
  * 
- * Converts the item's bounds to a bounding box in its parent's coordinate
+ * This function is intended to be used by subclasses of #GooCanvasItemSimple,
+ * typically in their get_requested_area() method.
+ *
+ * It converts the item's bounds to a bounding box in its parent's coordinate
  * space. If the item has no transformation matrix set then no conversion is
  * needed.
- *
- * This is typically needed when implementing the get_requested_area() method
- * for subclasses of #GooCanvasItemSimple.
  **/
 void
 goo_canvas_item_simple_user_bounds_to_parent (GooCanvasItemSimple *item,
@@ -1612,7 +1706,10 @@ goo_canvas_item_simple_user_bounds_to_parent (GooCanvasItemSimple *item,
  * @cr: a cairo context.
  * @pointer_events: specifies which parts of the path to check.
  * 
- * Checks if the given point is in the current path.
+ * This function is intended to be used by subclasses of #GooCanvasItemSimple.
+ *
+ * It checks if the given point is in the current path, using the item's
+ * style settings.
  * 
  * Returns: %TRUE if the given point is in the current path.
  **/
@@ -1652,6 +1749,29 @@ goo_canvas_item_simple_check_in_path (GooCanvasItemSimple   *item,
 }
 
 
+/**
+ * goo_canvas_item_simple_get_line_width:
+ * @item: a #GooCanvasItemSimple.
+ * 
+ * Gets the item's line width.
+ * 
+ * Returns: the item's line width.
+ **/
+gdouble
+goo_canvas_item_simple_get_line_width (GooCanvasItemSimple   *item)
+{
+  GValue *value;
+
+  value = goo_canvas_style_get_property (item->simple_data->style,
+					 goo_canvas_style_line_width_id);
+  if (value)
+    return value->data[0].v_double;
+  else if (item->canvas)
+    return goo_canvas_get_default_line_width (item->canvas);
+  else
+    return 2.0;
+}
+
 
 /**
  * SECTION:goocanvasitemmodelsimple
@@ -1674,6 +1794,7 @@ goo_canvas_item_simple_check_in_path (GooCanvasItemSimple   *item,
 
 
 static void item_model_interface_init  (GooCanvasItemModelIface *iface);
+static void goo_canvas_item_model_simple_dispose  (GObject *object);
 static void goo_canvas_item_model_simple_finalize (GObject *object);
 static void goo_canvas_item_model_simple_get_property (GObject              *object,
 						       guint                 prop_id,
@@ -1695,6 +1816,7 @@ goo_canvas_item_model_simple_class_init (GooCanvasItemModelSimpleClass *klass)
 {
   GObjectClass *gobject_class = (GObjectClass*) klass;
 
+  gobject_class->dispose  = goo_canvas_item_model_simple_dispose;
   gobject_class->finalize = goo_canvas_item_model_simple_finalize;
 
   gobject_class->get_property = goo_canvas_item_model_simple_get_property;
@@ -1714,12 +1836,21 @@ goo_canvas_item_model_simple_init (GooCanvasItemModelSimple *smodel)
 
 
 static void
-goo_canvas_item_model_simple_finalize (GObject *object)
+goo_canvas_item_model_simple_dispose (GObject *object)
 {
   GooCanvasItemModelSimple *smodel = (GooCanvasItemModelSimple*) object;
 
   goo_canvas_item_simple_free_data (&smodel->simple_data);
 
+  G_OBJECT_CLASS (goo_canvas_item_model_simple_parent_class)->dispose (object);
+}
+
+
+static void
+goo_canvas_item_model_simple_finalize (GObject *object)
+{
+  /*GooCanvasItemModelSimple *smodel = (GooCanvasItemModelSimple*) object;*/
+
   G_OBJECT_CLASS (goo_canvas_item_model_simple_parent_class)->finalize (object);
 }
 
@@ -1745,13 +1876,16 @@ goo_canvas_item_model_simple_get_property (GObject              *object,
       g_value_set_string (value, smodel->description);
       break;
     default:
-      goo_canvas_item_simple_get_common_property (object, simple_data, prop_id,
-						  value, pspec);
+      goo_canvas_item_simple_get_common_property (object, simple_data, NULL,
+						  prop_id, value, pspec);
       break;
     }
 }
 
 
+extern void _goo_canvas_item_model_emit_changed (GooCanvasItemModel *model,
+						 gboolean            recompute_bounds);
+
 static void
 goo_canvas_item_model_simple_set_property (GObject              *object,
 					   guint                 prop_id,
@@ -1790,7 +1924,7 @@ goo_canvas_item_model_simple_set_property (GObject              *object,
 								     prop_id,
 								     value,
 								     pspec);
-      g_signal_emit_by_name (smodel, "changed", recompute_bounds);
+      _goo_canvas_item_model_emit_changed (model, recompute_bounds);
       break;
     }
 }
@@ -1813,17 +1947,24 @@ goo_canvas_item_model_simple_set_parent (GooCanvasItemModel   *model,
 }
 
 
-static cairo_matrix_t*
-goo_canvas_item_model_simple_get_transform (GooCanvasItemModel     *model)
+static gboolean
+goo_canvas_item_model_simple_get_transform (GooCanvasItemModel  *model,
+					    cairo_matrix_t      *matrix)
 {
   GooCanvasItemModelSimple *smodel = (GooCanvasItemModelSimple*) model;
-  return smodel->simple_data.transform;
+  GooCanvasItemSimpleData *simple_data = &smodel->simple_data;
+
+  if (!simple_data->transform)
+    return FALSE;
+
+  *matrix = *simple_data->transform;
+  return TRUE;
 }
 
 
 static void
-goo_canvas_item_model_simple_set_transform (GooCanvasItemModel  *model,
-					    cairo_matrix_t      *transform)
+goo_canvas_item_model_simple_set_transform (GooCanvasItemModel   *model,
+					    const cairo_matrix_t *transform)
 {
   GooCanvasItemModelSimple *smodel = (GooCanvasItemModelSimple*) model;
   GooCanvasItemSimpleData *simple_data = &smodel->simple_data;
@@ -1841,7 +1982,7 @@ goo_canvas_item_model_simple_set_transform (GooCanvasItemModel  *model,
       simple_data->transform = NULL;
     }
 
-  g_signal_emit_by_name (smodel, "changed", TRUE);
+  _goo_canvas_item_model_emit_changed (model, TRUE);
 }
 
 
@@ -1875,7 +2016,7 @@ goo_canvas_item_model_simple_set_style (GooCanvasItemModel *model,
       simple_data->own_style = FALSE;
     }
 
-  g_signal_emit_by_name (smodel, "changed", TRUE);
+  _goo_canvas_item_model_emit_changed (model, TRUE);
 }
 
 
