@@ -24,12 +24,14 @@ G_BEGIN_DECLS
  *  path of the item, or %NULL.
  * @visibility_threshold: the threshold scale setting at which to show the item
  *  (if the @visibility setting is set to %VISIBLE_ABOVE_THRESHOLD).
- * @visibility: whether the item is visible, invisible, or visible above a
- *  given canvas scale setting.
- * @pointer_events: the events the item should receive.
+ * @visibility: the #GooCanvasItemVisibility setting specifying whether the
+ *  item is visible, invisible, or visible above a given canvas scale setting.
+ * @pointer_events: the #GooCanvasPointerEvents setting specifying the events
+ *  the item should receive.
  * @can_focus: if the item can take the keyboard focus.
  * @own_style: if the item has its own style, rather than using its parent's.
- * @clip_fill_rule: the fill rule used for the clip path.
+ * @clip_fill_rule: the #cairo_fill_rule_t setting specifying the fill rule
+ *  used for the clip path.
  *
  * This is the data common to both the model and view classes.
  */
@@ -39,12 +41,24 @@ struct _GooCanvasItemSimpleData
   GooCanvasStyle *style;
   cairo_matrix_t *transform;
   GArray *clip_path_commands;
+
+  /*< private >*/
+  /* We will store tooltips here in future. */
+  gchar *tooltip;
+
+  /*< public >*/
   gdouble visibility_threshold;
-  GooCanvasItemVisibility visibility	: 2;
-  GooCanvasPointerEvents pointer_events : 4;
+  guint visibility			: 2;
+  guint pointer_events			: 4;
   guint can_focus                       : 1;
   guint own_style                       : 1;
   guint clip_fill_rule			: 4;
+
+  /*< private >*/
+  /* We might use this in future for a cache setting - never/always/visible. */
+  guint cache_setting			: 3;
+  /* We might need this for tooltips in future. */
+  guint has_tooltip			: 1;
 };
 
 
@@ -90,6 +104,10 @@ struct _GooCanvasItemSimple
   GooCanvasBounds bounds;
   guint	need_update			: 1;
   guint need_entire_subtree_update      : 1;
+
+  /* <private> */
+  /* We might use this in future for things like a cache. */
+  gpointer priv;
 };
 
 /**
@@ -102,13 +120,13 @@ struct _GooCanvasItemSimple
  * @simple_update: subclasses should override this to calculate their new
  *  bounds, in user space.
  * @simple_paint: subclasses should override this to paint their item.
- * @simple_get_item_at: subclasses should override this to do hit-testing.
+ * @simple_is_item_at: subclasses should override this to do hit-testing.
  *
  * The #GooCanvasItemSimpleClass-struct struct contains several methods that
  * subclasses can override.
  *
  * Simple items need only implement the create_path() method. More complex
- * items must override the update(), paint() and get_item_at() methods.
+ * items must override the update(), paint() and is_item_at() methods.
  */
 struct _GooCanvasItemSimpleClass
 {
@@ -116,19 +134,19 @@ struct _GooCanvasItemSimpleClass
   GObjectClass parent_class;
 
   /*< public >*/
-  void		 (* simple_create_path)	(GooCanvasItemSimple *simple,
-					 cairo_t             *cr);
+  void		 (* simple_create_path)	(GooCanvasItemSimple   *simple,
+					 cairo_t               *cr);
 
-  void           (* simple_update)	(GooCanvasItemSimple *simple,
-					 cairo_t             *cr);
-  void           (* simple_paint)	(GooCanvasItemSimple *simple,
-					 cairo_t             *cr,
-					 GooCanvasBounds     *bounds);
-  GooCanvasItem* (* simple_get_item_at) (GooCanvasItemSimple *simple,
-					 gdouble              x,
-					 gdouble              y,
-					 cairo_t             *cr,
-					 gboolean             is_pointer_event);
+  void           (* simple_update)	(GooCanvasItemSimple   *simple,
+					 cairo_t               *cr);
+  void           (* simple_paint)	(GooCanvasItemSimple   *simple,
+					 cairo_t               *cr,
+					 const GooCanvasBounds *bounds);
+  gboolean       (* simple_is_item_at)  (GooCanvasItemSimple   *simple,
+					 gdouble                x,
+					 gdouble                y,
+					 cairo_t               *cr,
+					 gboolean               is_pointer_event);
 
   /*< private >*/
 
@@ -140,31 +158,32 @@ struct _GooCanvasItemSimpleClass
 };
 
 
-GType    goo_canvas_item_simple_get_type	   (void) G_GNUC_CONST;
+GType    goo_canvas_item_simple_get_type		(void) G_GNUC_CONST;
 
 
-void     goo_canvas_item_simple_get_path_bounds	   (GooCanvasItemSimple *item,
-						    cairo_t             *cr,
-						    GooCanvasBounds     *bounds);
-void     goo_canvas_item_simple_user_bounds_to_device (GooCanvasItemSimple *item,
-						       cairo_t             *cr,
-						       GooCanvasBounds     *bounds);
-void     goo_canvas_item_simple_user_bounds_to_parent (GooCanvasItemSimple *item,
-						       cairo_t             *cr,
-						       GooCanvasBounds     *bounds);
-gboolean goo_canvas_item_simple_check_in_path	   (GooCanvasItemSimple *item,
-						    gdouble              x,
-						    gdouble              y,
-						    cairo_t             *cr,
-						    GooCanvasPointerEvents pointer_events);
-void     goo_canvas_item_simple_paint_path	   (GooCanvasItemSimple *item,
-						    cairo_t             *cr);
+void     goo_canvas_item_simple_get_path_bounds		(GooCanvasItemSimple	*item,
+							 cairo_t		*cr,
+							 GooCanvasBounds	*bounds);
+void     goo_canvas_item_simple_user_bounds_to_device	(GooCanvasItemSimple	*item,
+							 cairo_t		*cr,
+							 GooCanvasBounds	*bounds);
+void     goo_canvas_item_simple_user_bounds_to_parent	(GooCanvasItemSimple	*item,
+							 cairo_t		*cr,
+							 GooCanvasBounds	*bounds);
+gboolean goo_canvas_item_simple_check_in_path		(GooCanvasItemSimple	*item,
+							 gdouble		 x,
+							 gdouble		 y,
+							 cairo_t		*cr,
+							 GooCanvasPointerEvents  pointer_events);
+void     goo_canvas_item_simple_paint_path		(GooCanvasItemSimple	*item,
+							 cairo_t		*cr);
 
-void     goo_canvas_item_simple_changed            (GooCanvasItemSimple *item,
-						    gboolean             recompute_bounds);
-void     goo_canvas_item_simple_check_style        (GooCanvasItemSimple *item);
-void	 goo_canvas_item_simple_set_model	   (GooCanvasItemSimple *item,
-						    GooCanvasItemModel  *model);
+void     goo_canvas_item_simple_changed			(GooCanvasItemSimple	*item,
+							 gboolean		 recompute_bounds);
+void     goo_canvas_item_simple_check_style		(GooCanvasItemSimple	*item);
+gdouble  goo_canvas_item_simple_get_line_width		(GooCanvasItemSimple   *item);
+void	 goo_canvas_item_simple_set_model		(GooCanvasItemSimple	*item,
+							 GooCanvasItemModel	*model);
 
 
 
