@@ -40,7 +40,9 @@ enum {
   PROP_TEXT,
   PROP_USE_MARKUP,
   PROP_ANCHOR,
-  PROP_ALIGN
+  PROP_ALIGN,
+  PROP_ELLIPSIZE,
+  PROP_WRAP
 };
 
 static void goo_canvas_text_finalize     (GObject            *object);
@@ -78,6 +80,22 @@ goo_canvas_text_install_common_properties (GObjectClass *gobject_class)
 							 FALSE,
 							 G_PARAM_READWRITE));
 
+  g_object_class_install_property (gobject_class, PROP_ELLIPSIZE,
+                                   g_param_spec_enum ("ellipsize",
+                                                      _("Ellipsize"),
+                                                      _("The preferred place to ellipsize the string, if the label does not have enough room to display the entire string"),
+						      PANGO_TYPE_ELLIPSIZE_MODE,
+						      PANGO_ELLIPSIZE_NONE,
+                                                      G_PARAM_READWRITE));
+
+  g_object_class_install_property (gobject_class, PROP_WRAP,
+                                   g_param_spec_enum ("wrap",
+                                                      _("Wrap"),
+                                                      _("The preferred method of wrapping the string if a width has been set"),
+						      PANGO_TYPE_WRAP_MODE,
+						      PANGO_WRAP_WORD,
+                                                      G_PARAM_READWRITE));
+
   /* Position */
   g_object_class_install_property (gobject_class, PROP_X,
 				   g_param_spec_double ("x",
@@ -127,6 +145,10 @@ goo_canvas_text_init (GooCanvasText *text)
   text->text_data = g_slice_new0 (GooCanvasTextData);
   text->text_data->width = -1.0;
   text->text_data->anchor = GTK_ANCHOR_NW;
+  text->text_data->ellipsize = PANGO_ELLIPSIZE_NONE;
+  text->text_data->wrap = PANGO_WRAP_WORD;
+
+  text->layout_width = -1.0;
 }
 
 
@@ -208,7 +230,9 @@ goo_canvas_text_finalize (GObject *object)
   GooCanvasItemSimple *simple = (GooCanvasItemSimple*) object;
   GooCanvasText *text = (GooCanvasText*) object;
 
-  if (!simple->model)
+  /* Free our data if we didn't have a model. (If we had a model it would
+     have been reset in dispose() and simple_data will be NULL.) */
+  if (simple->simple_data)
     {
       g_free (text->text_data->text);
       g_slice_free (GooCanvasTextData, text->text_data);
@@ -243,6 +267,12 @@ goo_canvas_text_get_common_property (GObject                 *object,
     case PROP_USE_MARKUP:
       g_value_set_boolean (value, text_data->use_markup);
       break;
+    case PROP_ELLIPSIZE:
+      g_value_set_enum (value, text_data->ellipsize);
+      break;
+    case PROP_WRAP:
+      g_value_set_enum (value, text_data->wrap);
+      break;
     case PROP_ANCHOR:
       g_value_set_enum (value, text_data->anchor);
       break;
@@ -294,6 +324,12 @@ goo_canvas_text_set_common_property (GObject                 *object,
     case PROP_USE_MARKUP:
       text_data->use_markup = g_value_get_boolean (value);
       break;
+    case PROP_ELLIPSIZE:
+      text_data->ellipsize = g_value_get_enum (value);
+      break;
+    case PROP_WRAP:
+      text_data->wrap = g_value_get_enum (value);
+      break;
     case PROP_ANCHOR:
       text_data->anchor = g_value_get_enum (value);
       break;
@@ -342,40 +378,53 @@ goo_canvas_text_create_layout (GooCanvasText     *text,
   GValue *svalue;
   PangoLayout *layout;
   PangoContext *context;
-  PangoRectangle logical_rect;
+  PangoRectangle ink_rect, logical_rect;
   double logical_width, logical_height, align_width, origin_x, origin_y;
   gchar *string;
+  double x1_extension, x2_extension, y1_extension, y2_extension;
+  cairo_font_options_t *font_options;
+  cairo_hint_metrics_t hint_metrics = CAIRO_HINT_METRICS_OFF;
 
   string = text_data->text ? text_data->text : "";
 
   layout = pango_cairo_create_layout (cr);
-
-  /* Set the font options to ensure the text layout is the same whatever
-     the scale is. */
   context = pango_layout_get_context (layout);
-  pango_cairo_context_set_font_options (context, GOO_CANVAS_TEXT_GET_CLASS (text)->font_options);
 
-  if (text_data->width > 0)
-    pango_layout_set_width (layout, (double) text_data->width * PANGO_SCALE);
+  if (text->layout_width > 0)
+    pango_layout_set_width (layout, (double) text->layout_width * PANGO_SCALE);
 
   if (text_data->use_markup)
     pango_layout_set_markup (layout, string, -1);
   else
     pango_layout_set_text (layout, string, -1);
 
-  svalue = goo_canvas_style_get_property (style, goo_canvas_style_font_desc_id);
+  svalue = goo_canvas_style_get_property (style,
+					  goo_canvas_style_font_desc_id);
   if (svalue)
     pango_layout_set_font_description (layout, svalue->data[0].v_pointer);
 
+  svalue = goo_canvas_style_get_property (style,
+					  goo_canvas_style_hint_metrics_id);
+  if (svalue)
+    hint_metrics = svalue->data[0].v_long;
+
+  font_options = cairo_font_options_create ();
+  cairo_font_options_set_hint_metrics (font_options, hint_metrics);
+  cairo_font_options_set_hint_style (font_options, CAIRO_HINT_STYLE_NONE);
+  pango_cairo_context_set_font_options (context, font_options);
+  cairo_font_options_destroy (font_options);
+
   if (text_data->alignment != PANGO_ALIGN_LEFT)
     pango_layout_set_alignment (layout, text_data->alignment);
 
-  /* FIXME: Sometimes we should be using the ink_rect rather than the
-     logical rect, e.g. for the actual bounds of the item view. */
+  pango_layout_set_ellipsize (layout, text_data->ellipsize);
+
+  pango_layout_set_wrap (layout, text_data->wrap);
+
   if (bounds)
     {
       /* Get size of the text, so we can position it according to anchor. */
-      pango_layout_get_extents (layout, NULL, &logical_rect);
+      pango_layout_get_extents (layout, &ink_rect, &logical_rect);
 
       logical_width = (double) logical_rect.width / PANGO_SCALE;
       logical_height = (double) logical_rect.height / PANGO_SCALE;
@@ -430,7 +479,7 @@ goo_canvas_text_create_layout (GooCanvasText     *text,
       if (origin_y_return)
 	*origin_y_return = origin_y;
 
-      /* Now calculate the bounds. */
+      /* Now calculate the logical bounds. */
       bounds->x1 = origin_x;
       bounds->y1 = origin_y;
 
@@ -455,6 +504,27 @@ goo_canvas_text_create_layout (GooCanvasText     *text,
 
       bounds->x2 = origin_x + logical_width;
       bounds->y2 = origin_y + logical_height;
+
+      /* Now adjust it to take into account the ink bounds. Calculate how far
+	 the ink rect extends outside each edge of the logical rect and adjust
+	 the bounds as necessary. */
+      x1_extension = logical_rect.x - ink_rect.x;
+      if (x1_extension > 0)
+	bounds->x1 -= x1_extension / PANGO_SCALE;
+
+      x2_extension = (ink_rect.x + ink_rect.width)
+	- (logical_rect.x + logical_rect.width);
+      if (x2_extension > 0)
+	bounds->x2 += x2_extension / PANGO_SCALE;
+
+      y1_extension = logical_rect.y - ink_rect.y;
+      if (y1_extension > 0)
+	bounds->y1 -= y1_extension / PANGO_SCALE;
+
+      y2_extension = (ink_rect.y + ink_rect.height)
+	- (logical_rect.y + logical_rect.height);
+      if (y2_extension > 0)
+	bounds->y2 += y2_extension / PANGO_SCALE;
     }
 
   return layout;
@@ -468,6 +538,11 @@ goo_canvas_text_update  (GooCanvasItemSimple *simple,
   GooCanvasText *text = (GooCanvasText*) simple;
   PangoLayout *layout;
 
+  /* Initialize the layout width to the text item's specified width property.
+     It may get changed later in get_requested_height() according to the
+     layout container and settings. */
+  text->layout_width = text->text_data->width;
+
   /* Compute the new bounds. */
   layout = goo_canvas_text_create_layout (text, cr, &simple->bounds,
 					  NULL, NULL);
@@ -490,31 +565,32 @@ goo_canvas_text_is_unpainted (GooCanvasStyle *style)
 }
 
 
-static GooCanvasItem*
-goo_canvas_text_get_item_at (GooCanvasItemSimple *simple,
-			     gdouble              x,
-			     gdouble              y,
-			     cairo_t             *cr,
-			     gboolean             is_pointer_event)
+static gboolean
+goo_canvas_text_is_item_at (GooCanvasItemSimple *simple,
+			    gdouble              x,
+			    gdouble              y,
+			    cairo_t             *cr,
+			    gboolean             is_pointer_event)
 {
   GooCanvasItemSimpleData *simple_data = simple->simple_data;
   GooCanvasText *text = (GooCanvasText*) simple;
-  GooCanvasItem *found_item = NULL;
   PangoLayout *layout;
   GooCanvasBounds bounds;
   PangoLayoutIter *iter;
-  PangoRectangle log_rect;
-  int px, py;
+  PangoRectangle ink_rect, log_rect;
+  int px, py, x1, y1, x2, y2;
+  int log_x2, ink_x2, log_y2, ink_y2;
   gdouble origin_x, origin_y;
+  gboolean in_item = FALSE;
 
   /* If there is no text just return. */
   if (!text->text_data->text || !text->text_data->text[0])
-    return NULL;
+    return FALSE;
 
   if (is_pointer_event
       && simple_data->pointer_events & GOO_CANVAS_EVENTS_PAINTED_MASK
       && goo_canvas_text_is_unpainted (simple_data->style))
-    return NULL;
+    return FALSE;
 
   layout = goo_canvas_text_create_layout (text, cr, &bounds,
 					  &origin_x, &origin_y);
@@ -528,12 +604,26 @@ goo_canvas_text_get_item_at (GooCanvasItemSimple *simple,
   iter = pango_layout_get_iter (layout);
   do
     {
-      pango_layout_iter_get_line_extents (iter, NULL, &log_rect);
+      pango_layout_iter_get_line_extents (iter, &ink_rect, &log_rect);
 
-      if (px >= log_rect.x && px < log_rect.x + log_rect.width 
-	  && py >= log_rect.y && py < log_rect.y + log_rect.height)
+      /* We use a union of the ink rect and the logical rect, as we want to
+	 let the user click on any part of the ink, even if it extends outside
+	 the character cell (i.e. the ink rect), or click on the space in the
+	 character cell (i.e. the logical rect). */
+      x1 = MIN (log_rect.x, ink_rect.x);
+      y1 = MIN (log_rect.y, ink_rect.y);
+
+      log_x2 = log_rect.x + log_rect.width;
+      ink_x2 = ink_rect.x + ink_rect.width;
+      x2 = MAX (log_x2, ink_x2);
+
+      log_y2 = log_rect.y + log_rect.height;
+      ink_y2 = ink_rect.y + ink_rect.height;
+      y2 = MAX (log_y2, ink_y2);
+
+      if (px >= x1 && px < x2 && py >= y1 && py < y2)
 	{
-	  found_item = (GooCanvasItem*) simple;
+	  in_item = TRUE;
 	  break;
 	}
 
@@ -543,14 +633,14 @@ goo_canvas_text_get_item_at (GooCanvasItemSimple *simple,
 
   g_object_unref (layout);
 
-  return found_item;
+  return in_item;
 }
 
 
 static void
-goo_canvas_text_paint (GooCanvasItemSimple *simple,
-		       cairo_t             *cr,
-		       GooCanvasBounds     *bounds)
+goo_canvas_text_paint (GooCanvasItemSimple   *simple,
+		       cairo_t               *cr,
+		       const GooCanvasBounds *bounds)
 {
   GooCanvasText *text = (GooCanvasText*) simple;
   PangoLayout *layout;
@@ -572,6 +662,55 @@ goo_canvas_text_paint (GooCanvasItemSimple *simple,
 }
 
 
+static gdouble
+goo_canvas_text_get_requested_height (GooCanvasItem	*item,
+				      cairo_t		*cr,
+				      gdouble            width)
+{
+  GooCanvasItemSimple *simple = (GooCanvasItemSimple*) item;
+  GooCanvasItemSimpleData *simple_data = simple->simple_data;
+  GooCanvasText *text = (GooCanvasText*) item;
+  PangoLayout *layout;
+  gdouble height;
+
+  /* If we have a transformation besides a simple scale & translation, just
+     return -1 as we can't adjust the height in that case. */
+  if (simple_data->clip_path_commands
+      || (simple_data->transform && (simple_data->transform->xy != 0.0
+				     || simple_data->transform->yx != 0.0)))
+    return -1;
+
+  cairo_save (cr);
+  if (simple_data->transform)
+    cairo_transform (cr, simple_data->transform);
+
+  /* Convert the width from the parent's coordinate space. Note that we only
+     need to support a simple scale operation here. */
+  text->layout_width = width;
+  if (simple_data->transform)
+    text->layout_width /= simple_data->transform->xx;
+
+  /* Create layout with given width. */
+  layout = goo_canvas_text_create_layout (text, cr, &simple->bounds,
+					  NULL, NULL);
+  g_object_unref (layout);
+
+  /* Convert to the parent's coordinate space. As above,  we only need to
+     support a simple scale operation here. */
+  height = simple->bounds.y2 - simple->bounds.y1;
+  if (simple_data->transform)
+    height *= simple_data->transform->yy;
+
+  /* Convert the item's bounds to device space. */
+  goo_canvas_item_simple_user_bounds_to_device (simple, cr, &simple->bounds);
+
+  cairo_restore (cr);
+
+  /* Return the new requested height of the text. */
+  return height;
+}
+
+
 static void
 goo_canvas_text_set_model    (GooCanvasItem      *item,
 			      GooCanvasItemModel *model)
@@ -595,7 +734,8 @@ goo_canvas_text_set_model    (GooCanvasItem      *item,
 static void
 canvas_item_interface_init (GooCanvasItemIface *iface)
 {
-  iface->set_model   = goo_canvas_text_set_model;
+  iface->get_requested_height = goo_canvas_text_get_requested_height;
+  iface->set_model            = goo_canvas_text_set_model;
 }
 
 
@@ -612,16 +752,9 @@ goo_canvas_text_class_init (GooCanvasTextClass *klass)
 
   simple_class->simple_update        = goo_canvas_text_update;
   simple_class->simple_paint         = goo_canvas_text_paint;
-  simple_class->simple_get_item_at   = goo_canvas_text_get_item_at;
+  simple_class->simple_is_item_at    = goo_canvas_text_is_item_at;
 
   goo_canvas_text_install_common_properties (gobject_class);
-
-  /* Create the font options once and reuse it. */
-  klass->font_options = cairo_font_options_create ();
-  cairo_font_options_set_hint_metrics (klass->font_options,
-				       CAIRO_HINT_METRICS_OFF);
-  cairo_font_options_set_hint_style (klass->font_options,
-				     CAIRO_HINT_STYLE_NONE);
 }
 
 
@@ -684,7 +817,10 @@ goo_canvas_text_model_class_init (GooCanvasTextModelClass *klass)
 static void
 goo_canvas_text_model_init (GooCanvasTextModel *tmodel)
 {
-
+  tmodel->text_data.width = -1.0;
+  tmodel->text_data.anchor = GTK_ANCHOR_NW;
+  tmodel->text_data.ellipsize = PANGO_ELLIPSIZE_NONE;
+  tmodel->text_data.wrap = PANGO_WRAP_WORD;
 }
 
 
@@ -729,15 +865,15 @@ goo_canvas_text_model_new (GooCanvasItemModel *parent,
 			   ...)
 {
   GooCanvasItemModel *model;
-  GooCanvasTextModel *emodel;
+  GooCanvasTextModel *tmodel;
   GooCanvasTextData *text_data;
   const char *first_property;
   va_list var_args;
 
   model = g_object_new (GOO_TYPE_CANVAS_TEXT_MODEL, NULL);
-  emodel = (GooCanvasTextModel*) model;
+  tmodel = (GooCanvasTextModel*) model;
 
-  text_data = &emodel->text_data;
+  text_data = &tmodel->text_data;
   text_data->text = g_strdup (string);
   text_data->x = x;
   text_data->y = y;
@@ -763,7 +899,9 @@ goo_canvas_text_model_new (GooCanvasItemModel *parent,
 static void
 goo_canvas_text_model_finalize (GObject *object)
 {
-  /*GooCanvasTextModel *emodel = (GooCanvasTextModel*) object;*/
+  GooCanvasTextModel *tmodel = (GooCanvasTextModel*) object;
+
+  g_free (tmodel->text_data.text);
 
   G_OBJECT_CLASS (goo_canvas_text_model_parent_class)->finalize (object);
 }
@@ -775,9 +913,9 @@ goo_canvas_text_model_get_property (GObject              *object,
 				    GValue               *value,
 				    GParamSpec           *pspec)
 {
-  GooCanvasTextModel *emodel = (GooCanvasTextModel*) object;
+  GooCanvasTextModel *tmodel = (GooCanvasTextModel*) object;
 
-  goo_canvas_text_get_common_property (object, &emodel->text_data, prop_id,
+  goo_canvas_text_get_common_property (object, &tmodel->text_data, prop_id,
 				       value, pspec);
 }
 
@@ -788,11 +926,11 @@ goo_canvas_text_model_set_property (GObject              *object,
 				    const GValue         *value,
 				    GParamSpec           *pspec)
 {
-  GooCanvasTextModel *emodel = (GooCanvasTextModel*) object;
+  GooCanvasTextModel *tmodel = (GooCanvasTextModel*) object;
 
-  goo_canvas_text_set_common_property (object, &emodel->text_data, prop_id,
+  goo_canvas_text_set_common_property (object, &tmodel->text_data, prop_id,
 				       value, pspec);
-  g_signal_emit_by_name (emodel, "changed", TRUE);
+  g_signal_emit_by_name (tmodel, "changed", TRUE);
 }
 
 
